package gymcard

import (
	"encoding/json"
	"errors"
	"net/http"
	"spaceofcheng/services/controllers"
	gym "spaceofcheng/services/models/gymcard"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/gymcard"
	"spaceofcheng/services/utils"
)

// GymcardController 信息
type GymcardController struct {
	dao *gymcard.GymcardServ
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *GymcardController) Constructor() {
	c.dao = gymcard.NewGymcardServ(c.Context)
}

// GetGymcardList 后台获取游泳健身卡列表
func (c *GymcardController) GetGymcardList() {
	caseid := c.GetString("caseid")
	page, _ := c.GetInt("page")
	pagesize, _ := c.GetInt("pagesize")
	if page < 0 {
		page = 1
	}

	if pagesize <= 0 {
		pagesize = 10
	}
	list, total, err := c.dao.GetGymcardList(caseid, page, pagesize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(map[string]interface{}{
		"list":     list,
		"page":     page,
		"pagesize": pagesize,
		"pagenum":  total,
	})
}

// GetGymcardListByCustomerId 根据客户id获取C端客户所有的健身卡
func (c *GymcardController) GetGymcardListByCustomerId() {
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")
	list, err := c.dao.GetCustomerGymBycustomerId(page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

// GetGymcardDetailbyId 根据游泳健身卡ID获取游泳健身卡详情
func (c *GymcardController) GetGymcardDetailbyId() {
	id := c.GetString(":gymcardId")
	gymcard, err := c.dao.GetGymcardDetailById(id)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(gymcard)
}

// GetCustomerGymDetailById 根据客户健身卡ID获取客户健身卡详情
func (c *GymcardController) GetCustomerGymDetailById() {
	id := c.GetString(":customerGymId")
	customerGym, err := c.dao.GetCustomerGymDetailById(id)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(customerGym)
}

// GetCustomerGymByPhone 根据客户手机获取客户可用的游泳健身卡
func (c *GymcardController) GetCustomerGymByPhone() {
	phone := c.GetString(":phone")
	customerGym, err := c.dao.GetCustomerGymByPhone(phone)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(customerGym)
}

// SaveGymcard 保存游泳健身卡
func (c *GymcardController) SaveGymcard() {
	gymCard := gym.Gymcard{}
	jsnStr := c.GetString("info")
	if jsnStr == "" {
		c.ResponseError(errors.New("未接收到保存内容"))
	}
	if err := json.Unmarshal([]byte(jsnStr), &gymCard); err != nil {
		utils.LogError("接收卡信息失败: " + err.Error())
		c.ResponseError(errors.New("接收参数失败"), http.StatusInternalServerError)
	}
	newGymcard, err := c.dao.SaveGymcard(gymCard)

	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(newGymcard)
}

// SendGymToCustomer 客户领卡
func (c *GymcardController) SendGymToCustomer() {
	gymcardId := c.GetString(":gymcardId")
	sysuserId := c.GetString(":sysuserId")
	customerGym, err := c.dao.SendGymCardToUser(gymcardId, sysuserId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(customerGym)
}

// // SendGymToCustomer 后台发卡给客户
// func (c *GymcardController) BackSendGymToCustomer() {
// 	gymcardId := c.GetString(":gymcardId")
// 	customerId := c.GetString(":customerId")
// 	sysUser := c.Context.Get("user").(model.SysUser)
// 	userId := sysUser.UserId
// 	customerGym, err := c.dao.BackSendGymCardToUser(gymcardId, userId, customerId)
// 	if err != nil {
// 		c.ResponseError(err)
// 	}
// 	c.ResponseJSON(customerGym)
// }

// VerifyCustomerGymcard 核销客户健身卡
func (c *GymcardController) VerifyCustomerGymcard() {
	cases := c.Context.Get("cases").([]model.SysUserCase)
	caseids := c.GetCaseIDs(cases)
	customerGymId := c.GetString(":customerGymId")
	err := c.dao.UpdateCustomerGym(customerGymId, caseids)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("核销成功")
}

// GiveGymToCustomer 赠送健身卡给客户
func (c *GymcardController) GiveGymToCustomer() {
	id := c.GetString(":id")
	if id == "" {
		c.ResponseError(errors.New("未指定卡"))
	}

	uids := c.GetString(":users")
	if uids == "" {
		c.ResponseError(errors.New("未指定赠送人"))
	}
	err := c.dao.GiveCardToCustomer(id, uids)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("赠送成功！")
}
