package luckdraw

import (
	"encoding/json"
	"errors"
	"net/http"
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/customer"
	luckdrawModel "spaceofcheng/services/models/luckdraw"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/luckdraw"
	"spaceofcheng/services/utils"
)

// LuckdrawController 应用
type LuckDrawController struct {
	serv    *luckdraw.LuckdrawServ
	custDAO *customer.CustomerDAO
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *LuckDrawController) Constructor() {
	c.serv = luckdraw.NewLuckdrawServ(c.Context)
	c.custDAO = customer.NewCustomerDAO(c.Context)
}

// GetLuckDrawTpl 获取抽奖模板
func (c *LuckDrawController) GetLuckDrawTpl() {
	tpls, err := c.serv.GetLuckDrawTpl()
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(tpls)
}

// GetLuckDrawList 获取抽奖列表
func (c *LuckDrawController) GetLuckDrawList() {
	caseids := c.GetString("caseid")
	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")
	name := c.GetString("name")
	status := c.GetString("status")
	list, err := c.serv.GetLuckDrawList(caseids, name, status, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

// GetLuckDrawRecordList 获取抽奖记录列表
func (c *LuckDrawController) GetLuckDrawRecordList() {
	luckdrawid := c.GetString(":luckdrawid")
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")
	tel := c.GetString("tel")
	list, err := c.serv.GetLuckDrawRecordList(luckdrawid, tel, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

// GetLuckDrawShareList 获取抽奖分享记录列表
func (c *LuckDrawController) GetLuckDrawShareList() {
	luckdrawid := c.GetString(":luckdrawid")
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")
	tel := c.GetString("tel")
	list, err := c.serv.GetLuckDrawShareList(luckdrawid, tel, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

// GetLuckDrawByID 根据id获取抽奖信息
func (c *LuckDrawController) GetLuckDrawByID() {
	id := c.GetString(":luckdrawid")
	luckdraw, err := c.serv.GetLuckDrawInfoByID(id)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(luckdraw)
}

// SaveLuckDraw 保存抽奖信息
func (c *LuckDrawController) SaveLuckDraw() {
	jsnStr := c.GetString("info")
	if jsnStr == "" {
		c.ResponseError(errors.New("未接收到保存内容"))
	}
	info := luckdrawModel.LuckDrawInfo{}
	if err := json.Unmarshal([]byte(jsnStr), &info); err != nil {
		utils.LogError("抽奖数据转换失败: " + err.Error())
		c.ResponseError(err)
	}
	newinfo, err := c.serv.SaveLuckDraw(&info)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(newinfo)
}

// DelLuckDraw 删除抽奖信息
func (c *LuckDrawController) DelLuckDraw() {
	luckdrawid := c.GetString(":luckdrawid")
	err := c.serv.DelLuckDraw(luckdrawid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("操作成功！")
}

// OpenLuckDraw 抽奖启用
func (c *LuckDrawController) OpenLuckDraw() {
	luckdrawid := c.GetString(":luckdrawid")
	err := c.serv.OpenLuckDraw(luckdrawid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("操作成功！")
}

// StopLuckDraw 抽奖启用
func (c *LuckDrawController) StopLuckDraw() {
	luckdrawid := c.GetString(":luckdrawid")
	err := c.serv.StopLuckDraw(luckdrawid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("操作成功！")
}

// GetUserByCode 根据code获取用户信息
func (c *LuckDrawController) GetUserByCode() {
	userRaw := c.Context.Get("customer")
	if userRaw == nil {
		c.ResponseError(errors.New("系统内部错误"), http.StatusInternalServerError)
	}

	user := userRaw.(model.TaCustomer)
	luckdrawid := c.GetString(":id")
	record, set, err := c.serv.GetUserLuckDrawByLuckDraw(user, luckdrawid)
	if err != nil {
		utils.LogError("获取用户抽奖信息失败: " + err.Error())
		c.ResponseError(errors.New("获取用户抽奖信息失败"))
	}
	var detail *model.TaPrizeDetail
	if record != nil {
		detail, err = c.serv.GetDetailByRecord(record.Id)
		if err != nil {
			utils.LogError("获取用户抽奖信息失败: " + err.Error())
			c.ResponseError(errors.New("获取用户抽奖信息失败"))
		}
	}

	c.ResponseJSON(map[string]interface{}{
		"user":   user,
		"record": record,
		"detail": detail,
		"set":    set,
	})
}

// GetLuckDraw 获取抽奖信息
func (c *LuckDrawController) GetLuckDraw() {
	id := c.GetString(":id")
	luckdraw, err := c.serv.GetLuckDraw(id)
	if err != nil {
		utils.LogError("获取抽奖信息失败: " + err.Error())
		c.ResponseError(errors.New("获取抽奖信息失败"))
	}
	c.ResponseJSON(luckdraw)
}

// LuckDraw 抽奖
func (c *LuckDrawController) LuckDraw() {
	id := c.GetString(":id")
	userRaw := c.Context.Get("customer")
	if userRaw == nil {
		c.ResponseError(errors.New("系统内部错误"), http.StatusInternalServerError)
	}

	user := userRaw.(model.TaCustomer)

	c.Context.Set("clienturl", c.Configer[controllers.AppConf].String("luckdrawNoticeURL"))
	c.Context.Set("tplid", c.Configer[controllers.WeChatConf].String("luckdrawMessageTplID"))

	prize, detail, err := c.serv.LuckDraw(id, &user)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(map[string]interface{}{
		"prize":  prize,
		"detail": detail,
	})
}

// GetRecordByLuckDraw 获取抽奖记录
func (c *LuckDrawController) GetRecordByLuckDraw() {
	id := c.GetString("id")
	records, err := c.serv.GetRecordByLuckDraw(id)
	if err != nil {
		utils.LogError("获取抽奖记录失败: " + err.Error())
		c.ResponseError(errors.New("获取抽奖记录失败"))
	}
	c.ResponseJSON(records)
}

// GetUserLuckDraw 获取用户的抽奖记录
func (c *LuckDrawController) GetUserLuckDraw() {
	userRaw := c.Context.Get("customer")
	if userRaw == nil {
		c.ResponseError(errors.New("系统内部错误"), http.StatusInternalServerError)
	}

	user := userRaw.(model.TaCustomer)

	records, err := c.serv.GetUserLuckDraw(user.CustomerId)
	if err != nil {
		utils.LogError("获取用户抽奖记录失败: " + err.Error())
		c.ResponseError(errors.New("获取用户抽奖记录失败"))
	}
	c.ResponseJSON(records)
}

// GetRecordByID 根据ID获取中奖记录信息
func (c *LuckDrawController) GetRecordByID() {
	id := c.GetString(":id")
	record, err := c.serv.GetRecordByID(id)
	if err != nil {
		utils.LogError("获取中奖记录失败: " + err.Error())
		c.ResponseError(errors.New("获取中奖记录失败"))
	}

	caseAddress, err := c.serv.GetCaseAddressByLuckDarw(record.LuckdrawId)
	if err != nil {
		utils.LogError("获取案场信息失败: " + err.Error())
		c.ResponseError(errors.New("获取案场信息失败"))
	}

	if caseAddress == "" {
		c.ResponseError(errors.New("无案场, 或案场地址没有维护"))
	}

	prize, err := c.serv.GetPrizeByID(record.PrizeId)
	if err != nil {
		utils.LogError("校验奖品记录失败: " + err.Error())
		c.ResponseError(errors.New("校验奖品记录失败"))
	}

	c.ResponseJSON(map[string]interface{}{
		"record":      record,
		"prize":       prize,
		"caseAddress": caseAddress,
	})
}

func (c *LuckDrawController) SaveShareRecord() {
	errIgnore := "error-ignore"

	id := c.GetString(":id")

	fromID := c.GetString("from")
	if fromID == "" {
		c.ResponseData(
			errIgnore,
			"没有分享人ID",
			http.StatusBadRequest,
		)
	}

	from, err := c.custDAO.GetCustomerByID(fromID)
	if err != nil {
		utils.LogError("获取分享人信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"获取分享人信息失败",
			http.StatusBadRequest,
		)
	}

	if from == nil || from.CustomerId == "" {
		c.ResponseData(
			errIgnore,
			"分享人信息不存在",
			http.StatusBadRequest,
		)
	}

	current := c.Context.Get("customer").(model.TaCustomer)

	// 自己分享给自己
	if from.CustomerId == current.CustomerId {
		c.ResponseData(
			errIgnore,
			"本人分享内容无效",
			http.StatusBadRequest,
		)
	}

	luckdraw, err := c.serv.GetLuckDrawByID(id)
	if err != nil {
		utils.LogError("获取抽奖信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"获取抽奖信息失败",
			http.StatusBadRequest,
		)
	}

	if luckdraw == nil {
		utils.LogError("获取抽奖信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"没有找到 id ("+id+") 对应的抽奖信息",
			http.StatusBadRequest,
		)
	}

	existsRec, err := c.serv.GetLuckDrawShareData(from.CustomerId, current.CustomerId, luckdraw.Id, luckdraw.CaseId)
	if err != nil {
		utils.LogError("获取已有分享信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"获取已有分享信息失败",
			http.StatusBadRequest,
		)
	}

	if existsRec != nil && len(existsRec) > 0 {
		c.ResponseData(
			errIgnore,
			"已分享记录, 再次分享无效",
			http.StatusBadRequest,
		)
	}

	shareDt := model.TaShareLuckyRecord{
		FromCustomerId:     from.CustomerId,
		FromCustomerName:   from.CustomerName,
		FromCustomerWxname: from.Name,
		FromCustomerTel:    from.Phone,
		ToCustomerId:       current.CustomerId,
		ToCustomerName:     current.CustomerName,
		ToCustomerWxname:   current.Name,
		ToCustomerTel:      current.Phone,
		CaseId:             luckdraw.CaseId,
		LuckydrawId:        luckdraw.Id,
		LuckydrawName:      luckdraw.Name,
	}

	if err := c.serv.SaveLuckDrawShareData(&shareDt); err != nil {
		utils.LogError("保存分享记录失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"保存分享记录失败",
			http.StatusBadRequest,
		)
	}

	fromIsNew := c.GetString("isnew")

	// 分享可以增加抽奖次数
	if luckdraw.ShareType == models.SHARETYPE_NEWUSER && fromIsNew == "true" {
		if err := c.serv.UpdateShareNum(luckdraw, fromID); err != nil {
			utils.LogError("更新抽奖次数失败: " + err.Error())
			c.ResponseData(
				errIgnore,
				"保存分享记录失败",
				http.StatusBadRequest,
			)
		}
	}

	c.ResponseJSON("ok")
}
