package controllers

import (
	"bytes"
	"encoding/json"
	"errors"
	"io/ioutil"
	"net/http"
	"net/url"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/customer"
	"spaceofcheng/services/utils"
	"strings"

	"github.com/astaxie/beego"
)

// UserController 用户
type WechatController struct {
	BaseController
	custServ *customer.CustomerServ
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *WechatController) Constructor() {
	c.custServ = customer.NewCustomerServ(c.Context)

	org := c.Context.Get("org").(model.SysOrg)
	cert, err := models.GetWeChatConfig(org.OrgId, models.WECHAT_WX)
	if err != nil {
		utils.LogError("获取微信相关配置失败: " + err.Error())
	} else {
		utils.WxClientSingleton(org.OrgId, cert)
	}
}

// WechatInfo 微信接入
func (c *WechatController) WechatInfo() {
	method := c.Ctx.Input.Method()
	if method == http.MethodGet {
		echostr := c.GetString("echostr")
		c.ResponseRaw([]byte(echostr))
	} else {
		c.WxReceive()
	}
}

// WxReceive
func (c *WechatController) WxReceive() {
	org := c.Context.Get("org").(model.SysOrg)
	wechat := utils.WxClientFor(org.OrgId)
	val, err := wechat.TransformMessage(string(c.Ctx.Input.RequestBody))
	if err != nil {
		utils.LogError("读取微信服务发送内容失败: " + err.Error())
		c.ResponseRaw([]byte(""))
	}

	// 如果是事件
	if val["MsgType"] == "event" {

		// 如果是扫描二维码
		qrcodePrefix := beego.AppConfig.String("qrcodePrefix")
		if (val["Event"] == "subscribe" && strings.Contains(val["EventKey"], "qrscene_"+qrcodePrefix)) || val["Event"] == "SCAN" {
			openID := val["FromUserName"]
			messagetxt, err := wechat.ResponseMessageText(openID, beego.AppConfig.String("followText"))
			if err != nil {
				utils.LogError("转换反馈微信消息失败: " + err.Error())
				c.ResponseRaw([]byte(""))
			}

			// 校验用户 并调用 悦奇接口
			// 使用 goroutine 主要是为了实现快速反馈微信
			// 微信只有 5 秒等待时间
			var done = make(chan bool)
			go func() {
				//
				user, err := c.checkUser(val)
				if err != nil {
					done <- true
					return
				}

				//
				EquipmentID := strings.Split(val["EventKey"], qrcodePrefix)[1]
				c.replyToYueQi(user, EquipmentID)
				done <- true
			}()

			//
			c.ResponseRaw(messagetxt, false)
			<-done
			c.DestroyContext()
			c.StopRun()
		}
	}
	c.ResponseRaw([]byte(""))
}

// 校验用户
func (c *WechatController) checkUser(val map[string]string) (map[string]interface{}, error) {
	org := c.Context.Get("org").(model.SysOrg)
	wechat := utils.WxClientFor(org.OrgId)
	openID := val["FromUserName"]

	user, err := wechat.GetUserDetail(openID)
	if err != nil {
		utils.LogError("获取微信用户详情失败:" + err.Error())
		return nil, errors.New("获取微信用户详情失败")
	}
	// 悦奇设备字段
	user["wechataccount"] = openID
	user["wechatofficial"] = wechat.GetAppID()

	// 判断用户是否存在
	getUser, err := c.custServ.GetCustUserByOpenID(openID)
	if err != nil {
		utils.LogError("查询来源用户失败: ", err)
		return user, nil
	}

	if getUser == nil || getUser.CustomerId == "" {
		// 新增用户
		_, err := c.custServ.SaveNewCustomer(user, "", "", "", "")
		if err != nil {
			utils.LogError("用户保存失败：", err)
			return user, nil
		}
	}

	return user, nil
}

// 反馈消息给悦奇
func (c *WechatController) replyToYueQi(user map[string]interface{}, EquipmentID string) error {
	var data = map[string]interface{}{
		"EquipmentId":    EquipmentID,
		"WechatUserInfo": user,
		"WXID":           1,
	}

	apiPrefix := beego.AppConfig.String("postwechatInfo")

	b, err := json.Marshal(data)
	if err != nil {
		return utils.LogError("转换待发送数据失败: " + err.Error())
	}

	body := bytes.NewBuffer([]byte(b))
	res, err := http.Post(apiPrefix, "application/json;charset=utf-8", body)
	if err != nil {
		return utils.LogError("调用悦奇接口失败: " + err.Error())
	}

	result, err := ioutil.ReadAll(res.Body)
	defer res.Body.Close()
	if err != nil {
		return utils.LogError("发送消息至悦奇失败：", err)
	}

	var postResult struct {
		Status  bool
		Message string
	}

	if err = json.Unmarshal(result, &postResult); err != nil {
		return utils.LogError("发送消息至悦奇失败：", err)
	}

	if !postResult.Status {
		return utils.LogError("发送消息至悦奇失败：", postResult.Message)
	}

	return nil
}

// UnicodeEmojiCode 过滤表情
func UnicodeEmojiCode(s string) string {
	ret := ""
	rs := []rune(s)
	for i := 0; i < len(rs); i++ {
		if len(string(rs[i])) == 4 {
			// u := `[\u` + strconv.FormatInt(int64(rs[i]), 16) + `]`
			ret += ""
		} else {
			ret += string(rs[i])
		}
	}
	return ret
}

// GetWxJsSDKSignature 获取 微信 jssdk 签名
func (c *WechatController) GetWxJsSDKSignature() {
	org := c.Context.Get("org").(model.SysOrg)
	url, _ := url.QueryUnescape(c.GetString("url"))

	cli := utils.WxClientFor(org.OrgId)
	if cli == nil {
		c.ResponseError(errors.New("初始化微信服务失败"))
	}

	res := cli.GetJsTicketSignature(url)
	c.ResponseJSON(res)
}

// MiniLogin 获取小程序 OPENID
func (c *WechatController) MiniLogin() {
	org := c.Context.Get("org").(model.SysOrg)
	code := c.GetString("code")

	cli := utils.MiniClientFor(org.OrgId)
	if cli == nil {
		c.ResponseError(errors.New("初始化小程序服务失败"))
	}

	res, err := cli.GetOpenID(code)
	if err != nil {
		utils.LogError("获取用户信息失败: " + err.Error())
		c.ResponseError(errors.New("获取用户信息失败"))
	}

	c.ResponseJSON(res)
}

// DecodeMiniData 解密小程序信息
func (c *WechatController) DecodeMiniData() {
	org := c.Context.Get("org").(model.SysOrg)

	cli := utils.MiniClientFor(org.OrgId)
	if cli == nil {
		c.ResponseError(errors.New("初始化小程序服务失败"))
	}

	params, err := c.GetBodyData()
	if err != nil {
		utils.LogError("url数据解码失败: " + err.Error())
		c.ResponseError(errors.New("url数据解码失败"))
	}

	data, err := url.QueryUnescape(params["encrypted_data"].(string))
	key, err := url.QueryUnescape(params["session_key"].(string))
	iv, err := url.QueryUnescape(params["iv"].(string))

	res, err := cli.GetUserFromEncryptData(data, key, iv)
	if err != nil {
		utils.LogError("解密用户信息失败: " + err.Error())
		c.ResponseError(errors.New("解密用户信息失败"))
	}

	c.ResponseJSON(res)
}
