package card

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/card"
	"spaceofcheng/services/models/coupon"
	"spaceofcheng/services/models/course"
	"spaceofcheng/services/models/customer"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/models/system"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
	"strings"
	"time"

	"github.com/astaxie/beego"
	"github.com/yl10/kit/guid"
)

// CardServ 系统处理
type CardServ struct {
	ctx       *utils.Context
	dao       *card.CardDAO
	custDao   *customer.CustomerDAO
	userDao   *system.UserDAO
	courseDao *course.CourseDAO
	coupondao *coupon.CouponDAO
}

// NewCardServ 初始化
func NewCardServ(ctx *utils.Context) *CardServ {
	return &CardServ{
		ctx:       ctx,
		dao:       card.NewCardDAO(ctx),
		custDao:   customer.NewCustomerDAO(ctx),
		userDao:   system.NewUserDAO(ctx),
		courseDao: course.NewCourseDAO(ctx),
		coupondao: coupon.NewCouponDAO(ctx),
	}
}

// GetCardList 获取卡列表
func (s *CardServ) GetCardList(caseID, sendtype, usetype string, pagenavi ...int) ([]model.TaCouponCard, int64, error) {
	filters := []string{}
	if caseID != "" {
		if err := utils.NewAuthEngine(s.ctx).CheckCase(caseID); err != nil {
			return nil, 0, err
		}

		filters = []string{
			"case_id='" + caseID + "'",
		}
	} else {
		casesRaw := s.ctx.Get("cases")
		if casesRaw == nil {
			return nil, 0, errors.New("请设置过滤案场")
		}

		cases := casesRaw.([]model.SysUserCase)
		caseIDs := []string{}
		for _, cs := range cases {
			caseIDs = append(caseIDs, cs.CaseId)
		}

		filters = []string{
			"case_id in ('" + strings.Join(caseIDs, "','") + "')",
		}
	}
	if sendtype != "" {
		filters = append(filters, "send_type='"+sendtype+"'")
		if usetype != "" {
			switch usetype {
			case models.COUPONCARD_USETYPE_LUCKDRAW:
				filters = append(filters, "card_id not in (select resource_id from sys_activity_action where active_type='"+models.ActGiveCard+"') and used_count=0")
				break
			case models.COUPONCARD_USETYPE_SYS:
				filters = append(filters, "card_id not in (select coupon_card_id from ta_luckdraw_record where coupon_card_type='"+models.PRIZE_TYPE_CARD+"') and used_count=0")
				break
			}
		}
	}

	limit := utils.GetPageNaviLimit(pagenavi...)
	res, total, err := s.dao.GetCardList(filters, limit)
	if err != nil {
		utils.LogError("查询卡列表失败: " + err.Error())
		return nil, 0, errors.New("查询卡列表失败")
	}

	return res, total, nil
}

// EditCard 编辑卡 , 没有则新增
func (s *CardServ) EditCard(card *card.CardInfo) error {
	if card.CardId == "" {
		return s.SaveCard(card)
	} else {
		return s.UpdateCard(card)
	}
}

// SaveCard 保存卡
// 字段校验没有在此函数中设置
func (s *CardServ) SaveCard(cd *card.CardInfo) error {
	if cd.CaseId == "" {
		return errors.New("未指定案场")
	}

	cd.CardId = guid.NewGUIDString()

	// 保存主信息
	cardMain := cd.TaCouponCard
	org := s.ctx.Get("org").(model.SysOrg)
	cardMain.OrgId = org.OrgId
	if err := s.dao.AddCard(&cardMain); err != nil {
		utils.LogError("保存卡失败: " + err.Error())
		return errors.New("保存卡失败")
	}

	// 保存图片
	if err := s.dao.UpdateImages(cd.Images, cd.CardId); err != nil {
		utils.LogError("保存卡图片失败: " + err.Error())
		return errors.New("保存卡图片失败")
	}

	// 保存分享
	share := cd.Share
	share.CardId = cd.CardId
	if err := s.dao.SaveCardShare(share); err != nil {
		utils.LogError("保存卡分享失败: " + err.Error())
		return errors.New("保存卡分享失败")
	}

	// 保存关联内容
	if err := s.dao.UpdateTarget(cd.Targets, cd.CardId); err != nil {
		utils.LogError("保存卡关联内容失败: " + err.Error())
		return errors.New("保存卡关联内容失败")
	}

	// 保存渠道
	if cd.ChannelId != "" {
		if err := s.dao.SaveCardChannel(cd.ChannelId, cd.CardId); err != nil {
			utils.LogError("保存对应渠道失败: " + err.Error())
			return errors.New("保存对应渠道失败")
		}
	}

	return nil
}

// UpdateCard 更新卡
// 字段校验没有在此函数中设置
func (s *CardServ) UpdateCard(cd *card.CardInfo) error {
	if cd.CaseId == "" {
		return errors.New("未指定案场")
	}

	if cd.CardId == "" {
		return errors.New("未指定卡")
	}

	// 更新主信息
	cardMain := cd.TaCouponCard
	if err := s.dao.UpdateCard(&cardMain, []string{"card_name", "video_url", "cover_url"}); err != nil {
		utils.LogError("更新卡失败: " + err.Error())
		return errors.New("更新卡失败")
	}

	// 更新图片
	if err := s.dao.UpdateImages(cd.Images, cd.CardId); err != nil {
		utils.LogError("更新卡图片失败: " + err.Error())
		return errors.New("更新卡图片失败")
	}

	// 更新分享
	share := cd.Share
	share.CardId = cd.CardId
	cols := []string{"card_share_info", "card_use_rule", "card_use_instruction"}
	if err := s.dao.UpdateCardShare(share, cols); err != nil {
		utils.LogError("更新卡分享失败: " + err.Error())
		return errors.New("更新卡分享失败")
	}

	// 删除渠道
	if err := s.dao.DelCardChannel(cd.CardId); err != nil {
		utils.LogError("删除对应渠道信息失败: " + err.Error())
		return errors.New("删除对应渠道信息失败")
	}

	// 保存渠道
	if cd.ChannelId != "" {
		if err := s.dao.SaveCardChannel(cd.ChannelId, cd.CardId); err != nil {
			utils.LogError("保存对应渠道失败: " + err.Error())
			return errors.New("保存对应渠道失败")
		}
	}
	// 关联内容 规则 不允许更新
	return nil
}

// GiveCard 赠送卡券
func (s *CardServ) GiveCard(CardID string, uids []string) error {
	// 当前用户
	userRaw := s.ctx.Get("user")
	if userRaw == nil {
		return errors.New("当前人员信息不正确, 请重试")
	}
	user := userRaw.(model.SysUser)

	card, err := s.dao.GetCardByID(CardID)
	if err != nil {
		utils.LogError("获取卡失败: " + err.Error())
		return errors.New("校验卡失败")
	}

	// 案场
	if err := utils.NewAuthEngine(s.ctx).CheckCase(card.CaseId); err != nil {
		return err
	}

	// 卡必须状态正常
	if card == nil || card.CardId == "" || card.Status != models.STATUS_NORMAL {
		return errors.New("卡已失效或者不存在")
	}

	// 卡库存判断
	if card.IsOver == 0 && card.SentCount+len(uids) > card.TotalCount {
		return errors.New("卡库存不足")
	}

	now := time.Now().Local()
	if card.EndDate.Before(now) {
		return errors.New("卡已失效，不允许赠送")
	}

	for _, uid := range uids {
		// 获取客户信息
		cust, err := s.custDao.GetCustomerByID(uid)
		if err != nil {
			utils.LogError("查询客户信息失败: " + err.Error())
			return errors.New("校验客户信息失败")
		}

		if err := s.GiveCardTo(&user, cust, card); err != nil {
			return err
		}

	}

	return nil
}

// GiveCardTo 赠送卡
// 本方法会被多出调用, 方法内使用 Context 将是不安全的
func (s *CardServ) GiveCardTo(from *model.SysUser, to *model.TaCustomer, card *card.CardInfo) error {
	if from.UserId == "" {
		return errors.New("必须指定赠送人")
	}

	if to.CustomerId == "" {
		return errors.New("必须指定接收人")
	}

	// 卡必须状态正常
	if card.CardId == "" || card.Status != models.STATUS_NORMAL {
		return errors.New("卡已失效或者不存在")
	}

	// 计算卡券有效期
	startDate := card.StartDate
	endDate := card.EndDate

	// 检查用户是否已经有了该券
	// custcardS, err := s.dao.CheckCustCard(to.CustomerId, card.CardId, startDate, endDate)
	// if err != nil {
	// 	utils.LogError("校验客户卡失败: " + err.Error())
	// 	return errors.New("校验客户卡失败")
	// }

	// if custcardS != nil && len(custcardS) > 0 {
	// 	return errors.New("客户 " + to.CustomerName + " 已经拥有该券")
	// }

	// 放入我的卡券
	custcard := model.TaCustomerCard{
		CustomerCardId:   utils.GetGUID(),
		CardId:           card.CardId,
		CustomerCardName: card.CardName,
		CustomerId:       to.CustomerId,
		StartDate:        startDate,
		EndDate:          endDate,
		ReceiveDate:      time.Now().Local(),
		CaseId:           card.CaseId,
		OrgId:            card.OrgId,
		ReceivingType:    models.RECEIVEING_TYPE_GIVE,
		Price:            card.Price,
	}

	// 放入赠送记录
	rec := model.TaCouponGiveRecord{
		GiftType:   models.GIVE_GIFT_COUPON,
		GiftId:     card.CardId,
		GiftName:   card.CardName,
		FromId:     from.UserId,
		FromName:   from.RealName,
		ToId:       to.CustomerId,
		ToName:     to.CustomerName,
		CreateDate: time.Now().Local(),
	}

	// 入库
	if err := s.dao.SaveCustomerCards([]model.TaCustomerCard{custcard}); err != nil {
		utils.LogError("保存客户卡失败: " + err.Error())
		return errors.New("保存客户卡失败")
	}

	if err := s.coupondao.SaveCouponRecoreds([]model.TaCouponGiveRecord{rec}); err != nil {
		utils.LogError("保存卡赠送记录失败: " + err.Error())
		return errors.New("保存卡赠送记录失败")
	}

	// 加入我的课程信息
	err := s.SaveCustomerCourseByCard(&custcard)
	if err != nil {
		return err
	}

	// TODO
	// 券库存处理
	var cardinfo = model.TaCouponCard{
		CardId:    card.CardId,
		SentCount: card.SentCount + 1,
		UsedCount: card.UsedCount + 1,
	}
	cols := []string{
		"sent_count",
		"used_count",
	}
	if err := s.dao.UpdateCard(&cardinfo, cols); err != nil {
		utils.LogError("回填体验卡信息失败:", err)
		return errors.New("回填体验卡信息失败！")
	}
	return nil
}

// GetCardByID 获取卡
// 未进行权限校验, 非安全的
func (s *CardServ) GetCardByID(cardID string) (*card.CardInfo, error) {
	card, err := s.dao.GetCardByID(cardID)
	if err != nil {
		utils.LogError("获取卡失败: " + err.Error())
		return nil, errors.New("校验卡失败")
	}

	return card, nil
}

// GetCardByIDWithCheck 获取卡
func (s *CardServ) GetCardByIDWithCheck(cardID string) (*card.CardInfo, error) {
	card, err := s.dao.GetCardByID(cardID)
	if err != nil {
		utils.LogError("获取卡失败: " + err.Error())
		return nil, errors.New("校验卡失败")
	}

	if err := utils.NewAuthEngine(s.ctx).CheckCase(card.CaseId); err != nil {
		return nil, err
	}

	return card, nil
}

// GetCardWithCustomer 获取体验卡明细
func (s *CardServ) GetCardWithCustomer(cardid string) (*card.CardDetailWithSales, error) {
	if cardid == "" {
		return nil, errors.New("没有卡信息！")
	}
	cardDetail, err := s.dao.GetCardDetail(cardid)
	if err != nil {
		utils.LogError("获取体验卡失败: " + err.Error())
		return nil, errors.New("获取体验卡失败")
	}
	return cardDetail, nil
}

// GetCardByCustomer 获取我的体验卡
func (s *CardServ) GetCardByCustomer() ([]card.CustomerCardWithShare, error) {
	org := s.ctx.Get("org").(model.SysOrg)
	customer := s.ctx.Get("customer").(model.TaCustomer)
	cards, err := s.dao.GetCardByCustomer(org.OrgId, customer.CustomerId)
	if err != nil {
		utils.LogError("获取体验卡失败: " + err.Error())
		return nil, errors.New("获取体验卡失败")
	}
	return cards, nil
}

// GetCustomerCardByID 获取我的体验卡详情
func (s *CardServ) GetCustomerCardByID(id string) (*card.CustomerCardWithShare, error) {
	card, err := s.dao.GetCustomerCardByID(id)
	if err != nil {
		utils.LogError("获取体验卡失败: " + err.Error())
		return nil, errors.New("获取体验卡失败")
	}
	return card, nil
}

func (s *CardServ) GetSysCardList(caseid string) ([]model.TaCouponCard, error) {
	cards, err := s.dao.GetSysCardList(caseid)
	if err != nil {
		utils.LogError("获取体验卡失败: " + err.Error())
		return nil, errors.New("获取体验卡失败")
	}
	return cards, nil
}

// GetCaseUsableCard 获取案场可用卡
func (s *CardServ) GetCaseUsableCard(page, pageSize int) (map[string]interface{}, error) {

	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	customer := s.ctx.Get("customer").(model.TaCustomer)
	if customer.UserId == "" {
		return nil, errors.New("您没有权限查看数据")
	}
	userTypes, err := s.userDao.GetUserWithTypeByID(customer.UserId)
	if err != nil {
		utils.LogError("获取用户信息失败: " + err.Error())
		return nil, errors.New("获取用户信息失败")
	}
	if len(userTypes) == 0 {
		return nil, errors.New("您没有权限查看数据")
	}
	caseinfo, err := s.userDao.GetUserBelongCase(customer.UserId)
	if err != nil {
		return nil, errors.New("您没有案场信息！")
	}
	var userid = ""
	if userTypes[0].TypeId != models.USERTYPE_MANAGER {
		userid = customer.UserId
	}
	coupons, err := s.dao.GetCaseUsableCard(caseinfo.CaseId, userid, page, pageSize)
	if err != nil {
		return nil, errors.New("获取数据失败！")
	}
	total, err := s.dao.GetCaseUsableCardCount(caseinfo.CaseId, userid)
	if err != nil {
		utils.LogError("获取卡信息失败: " + err.Error())
		return nil, errors.New("获取卡信息失败")
	}
	return map[string]interface{}{
		"list":     coupons,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

// ReceiveCard 卡领取
func (s *CardServ) ReceiveCard(id, saleid, serialcode string) error {
	if saleid == "" || serialcode == "" {
		return errors.New("log-error-参数不全！不允许领取")
	}

	// 数据校验
	customer := s.ctx.Get("customer").(model.TaCustomer)
	if customer.Phone == "" {
		return errors.New("log-error-用户未绑定手机号！")
	}
	if customer.UserId != "" {
		return errors.New("log-error-不允许内部人员领取卡券！")
	}

	card, err := s.dao.GetCardInfoByID(id)
	if err != nil {
		utils.LogError("领取卡获取卡信息失败：", err)
		return errors.New("log-error-领取卡失败！")
	}
	if card.TotalCount <= card.SentCount && card.IsOver == 0 {
		return errors.New("log-error-卡券已领完！")
	}
	if card.EndDate.Before(time.Now()) {
		return errors.New("log-error-卡券已过期")
	}

	// 销售校验
	saler, err := s.custDao.GetUserByID(saleid)
	if err != nil {
		utils.LogError("领取卡获取销售信息失败：", err)
		return errors.New("log-error-领取卡失败！")
	}
	if saleid != beego.AppConfig.String("defaultShareUserID") {
		caseinfo, err := s.userDao.GetUserBelongCase(saleid)
		if err != nil {
			utils.LogError("领取卡获取销售案场信息失败：", err)
			return errors.New("log-error-领取卡失败！")
		}
		if caseinfo.CaseId != card.CaseId {
			return errors.New("log-error-销售不合法！")
		}
	}

	// 验证码判断
	isok, err := s.dao.CheckBySerialCode(card.CardId, serialcode)
	if err != nil {
		utils.LogError("获取卡批次码判断失败：", err)
		return errors.New("log-error-领取卡失败！")
	}
	if !isok {
		return errors.New("log-error-该卡已被领取！")
	}

	// 领取
	var customerCard = model.TaCustomerCard{
		CustomerCardId:   utils.GetGUID(),
		CardId:           card.CardId,
		CustomerCardName: card.CardName,
		CustomerId:       customer.CustomerId,
		SalesId:          saler.UserId,
		SalesName:        saler.UserName,
		StartDate:        card.StartDate,
		EndDate:          card.EndDate,
		CaseId:           card.CaseId,
		OrgId:            card.OrgId,
		SerialCode:       serialcode,
		ReceivingType:    models.RECEIVEING_TYPE_SALES,
		Price:            card.Price,
	}

	err = s.dao.SaveCustomerCard(&customerCard)
	if err != nil {
		utils.LogError("保存用户体验卡信息失败：", err)
		return errors.New("log-error-领取卡失败！")
	}
	// 加入我的课程信息
	err = s.SaveCustomerCourseByCard(&customerCard)
	if err != nil {
		return err
	}
	// 回填体验卡信息
	card.SentCount = card.SentCount + 1
	card.UsedCount = card.UsedCount + 1
	cols := []string{
		"sent_count",
		"used_count",
	}
	err = s.dao.UpdateCard(card, cols)
	if err != nil {
		utils.LogError("回填体验卡信息失败:", err)
		return errors.New("log-error-领取卡失败！")
	}
	return nil
}

// SaveCustomerCourseByCard 保存用户的课程信息
func (s *CardServ) SaveCustomerCourseByCard(customerCard *model.TaCustomerCard) error {
	customer, err := s.custDao.GetCustomerByID(customerCard.CustomerId)
	if err != nil {
		utils.LogError("获取客户信息失败:", err)
		return errors.New("获取客户信息失败")
	}
	card, err := s.dao.GetCardByID(customerCard.CardId)
	if err != nil {
		utils.LogError("获取体验卡信息失败:", err)
		return errors.New("领取卡失败！")
	}
	for _, target := range card.Targets {
		course, err := s.courseDao.GetCourseInfo(target.TargetId)
		if err != nil {
			utils.LogError("领取卡时课程获取失败:", err)
			return errors.New("领取卡时课程获取失败！")
		}
		// 我的课程信息
		custCourse := model.TaCustomerCourse{
			CourseId:         course.CourseId,
			CustomerId:       customer.CustomerId,
			OrgId:            card.OrgId,
			CaseId:           card.CaseId,
			CourseName:       course.CourseName,
			LocationId:       course.LocationId,
			Price:            course.Price,
			CourseNum:        course.CourseNum,
			JoinNum:          0,
			CreateDate:       time.Now().Local(),
			CourseObtaimType: models.COURSE_GETBY_CARD,
			SourceId:         customerCard.CustomerCardId,
			IsDone:           models.BOOL_FALSE,
		}

		custCourseDetails := make([]model.TaCustomerCourseDetail, 0)

		for _, subCourse := range course.CourseDetail {
			d2 := model.TaCustomerCourseDetail{
				CourseId:     course.CourseId,
				DetailId:     subCourse.DetailId,
				StartDate:    subCourse.BeginDate,
				EndDate:      subCourse.EndDate,
				VerifyStatus: models.VERIFY_USEABLE,
				CaseId:       customerCard.CaseId,
			}
			custCourseDetails = append(custCourseDetails, d2)
		}

		customerCourseId, err := s.courseDao.SaveCourseOfCustomer(&custCourse, custCourseDetails)
		if err != nil {
			utils.LogError("插入我的课程失败: " + err.Error())
			return errors.New("写入我的课程失败")
		}
		custQrcode := model.TaCustomerCourseQrcode{
			CustomerCourseId: customerCourseId,
			QrcodeType:       models.QRCODE_TYPE_COURSE,
			CreateDate:       time.Now(),
			CustomerQrcode:   utils.GenerateQRCode(),
			Status:           models.STATUS_NORMAL,
		}
		err = s.courseDao.SaveCustomerCourseQrcode(&custQrcode)
		if err != nil {
			utils.LogError("生成我的课程二维码失败：" + err.Error())
			return errors.New("生成我的课程二维码失败")
		}

		utils.SendSMS("cardget", customer.Phone, course.CourseName)
	}

	return nil
}

// ChannelReceiveCard 渠道端卡领取
func (s *CardServ) ChannelReceiveCard(id string) error {
	/// 数据校验
	customer := s.ctx.Get("customer").(model.TaCustomer)
	if customer.Phone == "" {
		return errors.New("log-error-用户未绑定手机号！")
	}
	if customer.UserId != "" {
		return errors.New("log-error-不允许内部人员领取卡券！")
	}

	card, err := s.dao.GetCardInfoByID(id)
	if err != nil {
		utils.LogError("领取卡获取卡信息失败：", err)
		return errors.New("log-error-领取卡失败！")
	}
	if card.TotalCount <= card.SentCount && card.IsOver == 0 {
		return errors.New("log-error-卡券已领完！")
	}
	if card.EndDate.Before(time.Now()) {
		return errors.New("log-error-卡券已过期")
	}

	// 重复性校验
	isok, err := s.dao.CheckByCustomer(id, customer.CustomerId)
	if err != nil {
		utils.LogError("判断是否已领取失败：", err)
		return errors.New("log-error-领取卡失败！")
	}

	if !isok {
		return errors.New("log-error-您已领过该卡！")
	}
	// 领取
	var customerCard = model.TaCustomerCard{
		CustomerCardId:   utils.GetGUID(),
		CardId:           card.CardId,
		CustomerCardName: card.CardName,
		CustomerId:       customer.CustomerId,
		StartDate:        card.StartDate,
		EndDate:          card.EndDate,
		CaseId:           card.CaseId,
		OrgId:            card.OrgId,
		ReceivingType:    models.RECEIVEING_TYPE_CHANNEL,
		Price:            card.Price,
	}

	err = s.dao.SaveCustomerCard(&customerCard)
	if err != nil {
		utils.LogError("保存用户体验卡信息失败：", err)
		return errors.New("log-error-领取卡失败！")
	}
	// 加入我的课程信息
	err = s.SaveCustomerCourseByCard(&customerCard)
	if err != nil {
		return err
	}
	// 回填体验卡信息
	card.SentCount = card.SentCount + 1
	card.UsedCount = card.UsedCount + 1
	cols := []string{
		"sent_count",
		"used_count",
	}
	err = s.dao.UpdateCard(card, cols)
	if err != nil {
		utils.LogError("回填体验卡信息失败:", err)
		return errors.New("log-error-领取卡失败！")
	}
	return nil
}

// GetCustomerCardByCustomerAndSale 根据用户与销售信息获取领取记录
func (s *CardServ) GetCustomerCardByCustomerAndSale(customerid string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	saler := s.ctx.Get("customer").(model.TaCustomer)
	if saler.UserId == "" {
		return nil, errors.New("您没有权限查看数据")
	}
	cards, err := s.dao.GetCustomerCardByCustomerAndSale(customerid, saler.UserId, page, pageSize)
	if err != nil {
		return nil, err
	}
	total, err := s.dao.GetCustomerCardByCustomerAndSaleCount(customerid, saler.UserId)
	if err != nil {
		utils.LogError("获取用户的领取记录失败: " + err.Error())
		return nil, errors.New("获取用户的领取记录失败")
	}
	return map[string]interface{}{
		"list":     cards,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}
