package coupon

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/coupon"
	"spaceofcheng/services/models/customer"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/models/system"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
	"strings"
	"time"

	"github.com/astaxie/beego"
	"github.com/yl10/kit/guid"
)

// CouponServ 系统处理
type CouponServ struct {
	ctx     *utils.Context
	dao     *coupon.CouponDAO
	custDao *customer.CustomerDAO
	userDao *system.UserDAO
}

// NewCouponServ 初始化
func NewCouponServ(ctx *utils.Context) *CouponServ {
	return &CouponServ{
		ctx:     ctx,
		dao:     coupon.NewCouponDAO(ctx),
		custDao: customer.NewCustomerDAO(ctx),
		userDao: system.NewUserDAO(ctx),
	}
}

// GetCouponList 获取优惠券列表
func (s *CouponServ) GetCouponList(caseID, sendtype, usetype string, pagenavi ...int) ([]model.TaCoupon, int64, error) {
	filters := []string{}

	if caseID != "" {
		if err := utils.NewAuthEngine(s.ctx).CheckCase(caseID); err != nil {
			return nil, 0, err
		}

		filters = []string{
			"case_id='" + caseID + "'",
		}
	} else {
		casesRaw := s.ctx.Get("cases")
		if casesRaw == nil {
			return nil, 0, errors.New("请设置过滤案场")
		}

		cases := casesRaw.([]model.SysUserCase)
		caseIDs := []string{}
		for _, cs := range cases {
			caseIDs = append(caseIDs, cs.CaseId)
		}

		filters = []string{
			"case_id in ('" + strings.Join(caseIDs, "','") + "')",
		}
	}
	if sendtype != "" {
		filters = append(filters, "send_type='"+sendtype+"'")
		if usetype != "" {
			switch usetype {
			case models.COUPONCARD_USETYPE_LUCKDRAW:
				filters = append(filters, "coupon_id not in (select resource_id from sys_activity_action where active_type='"+models.ActGiveCoupon+"') and used_count=0")
				break
			case models.COUPONCARD_USETYPE_SYS:
				filters = append(filters, "coupon_id not in (select coupon_card_id from ta_luckdraw_record where coupon_card_type='"+models.PRIZE_TYPE_COUPON+"') and used_count=0")
				break
			}
		}
	}

	limit := utils.GetPageNaviLimit(pagenavi...)
	res, total, err := s.dao.GetCouponList(filters, limit)
	if err != nil {
		utils.LogError("查询优惠券列表失败: " + err.Error())
		return nil, 0, errors.New("查询优惠券列表失败")
	}

	return res, total, nil
}

// EditCoupon 编辑优惠券 , 没有则新增
func (s *CouponServ) EditCoupon(cp *coupon.CouponInfo) error {
	if cp.CouponId == "" {
		return s.SaveCoupon(cp)
	} else {
		return s.UpdateCoupon(cp)
	}
}

// SaveCoupon 保存优惠券
// 字段校验没有在此函数中设置
func (s *CouponServ) SaveCoupon(cp *coupon.CouponInfo) error {
	if cp.CaseId == "" {
		return errors.New("未指定案场")
	}

	cp.CouponId = guid.NewGUIDString()

	// 保存主信息
	cpMain := cp.TaCoupon
	org := s.ctx.Get("org").(model.SysOrg)
	cpMain.OrgId = org.OrgId
	if err := s.dao.SaveCoupon(&cpMain); err != nil {
		utils.LogError("保存优惠券失败: " + err.Error())
		return errors.New("保存优惠券失败")
	}

	// 保存图片
	if err := s.dao.UpdateImages(cp.Images, cp.CouponId); err != nil {
		utils.LogError("保存优惠券图片失败: " + err.Error())
		return errors.New("保存优惠券图片失败")
	}

	// 保存分享
	share := cp.Share
	share.CouponId = cp.CouponId
	// cols := []string{"coupon_share_info", "use_rule", "use_instruction"}
	if err := s.dao.SaveCouponShare(share); err != nil {
		utils.LogError("保存优惠券分享失败: " + err.Error())
		return errors.New("保存优惠券分享失败")
	}

	// 保存关联内容
	if err := s.dao.UpdateTarget(cp.Targets, cp.CouponId); err != nil {
		utils.LogError("保存优惠券关联内容失败: " + err.Error())
		return errors.New("保存优惠券关联内容失败")
	}

	// 保存规则
	if err := s.dao.UpdateRule(cp.Rules, cp.CouponId); err != nil {
		utils.LogError("保存优惠券规则失败: " + err.Error())
		return errors.New("保存优惠券规则失败")
	}

	// 保存渠道
	if cp.ChannelId != "" {
		if err := s.dao.SaveCouponChannel(cp.ChannelId, cp.CouponId); err != nil {
			utils.LogError("保存对应渠道失败: " + err.Error())
			return errors.New("保存对应渠道失败")
		}
	}

	return nil
}

// UpdateCoupon 更新优惠券
// 字段校验没有在此函数中设置
func (s *CouponServ) UpdateCoupon(cp *coupon.CouponInfo) error {
	if cp.CaseId == "" {
		return errors.New("未指定案场")
	}

	if cp.CouponId == "" {
		return errors.New("未指定优惠券")
	}

	// 更新主信息
	cpMain := cp.TaCoupon
	if err := s.dao.UpdateCoupon(&cpMain, []string{"coupon_name", "cover_url", "video_url"}); err != nil {
		utils.LogError("更新优惠券失败: " + err.Error())
		return errors.New("更新优惠券失败")
	}

	// 更新图片
	if err := s.dao.UpdateImages(cp.Images, cp.CouponId); err != nil {
		utils.LogError("更新优惠券图片失败: " + err.Error())
		return errors.New("更新优惠券图片失败")
	}

	// 更新分享
	share := cp.Share
	share.CouponId = cp.CouponId
	cols := []string{"coupon_share_info", "use_rule", "use_instruction"}
	if err := s.dao.UpdateCouponShare(share, cols); err != nil {
		utils.LogError("更新优惠券分享失败: " + err.Error())
		return errors.New("更新优惠券分享失败")
	}

	// 保存关联内容
	if err := s.dao.UpdateTarget(cp.Targets, cp.CouponId); err != nil {
		utils.LogError("保存优惠券关联内容失败: " + err.Error())
		return errors.New("保存优惠券关联内容失败")
	}

	// 保存规则
	if err := s.dao.UpdateRule(cp.Rules, cp.CouponId); err != nil {
		utils.LogError("保存优惠券规则失败: " + err.Error())
		return errors.New("保存优惠券规则失败")
	}

	// 删除渠道
	if err := s.dao.DelCouponChannel(cp.CouponId); err != nil {
		utils.LogError("删除对应渠道信息失败: " + err.Error())
		return errors.New("删除对应渠道信息失败")
	}

	// 保存渠道
	if cp.ChannelId != "" {
		if err := s.dao.SaveCouponChannel(cp.ChannelId, cp.CouponId); err != nil {
			utils.LogError("保存对应渠道失败: " + err.Error())
			return errors.New("保存对应渠道失败")
		}
	}
	return nil
}

// GiveCoupon 赠送卡券
func (s *CouponServ) GiveCoupon(couponID string, uids []string) error {
	// 当前用户
	userRaw := s.ctx.Get("user")
	if userRaw == nil {
		return errors.New("当前人员信息不正确, 请重试")
	}
	user := userRaw.(model.SysUser)

	cp, err := s.dao.GetCouponByID(couponID)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return errors.New("校验优惠券失败")
	}

	// 案场
	if err := utils.NewAuthEngine(s.ctx).CheckCase(cp.CaseId); err != nil {
		return err
	}

	// 优惠券必须状态正常
	if cp == nil || cp.CouponId == "" || cp.Status != models.STATUS_NORMAL {
		return errors.New("优惠券已失效或者不存在")
	}

	// 卡库存判断
	if cp.IsOver == 0 && cp.SentCount+len(uids) > cp.TotalCount {
		return errors.New("优惠券库存不足")
	}

	now := time.Now().Local()
	if cp.ValidDays <= 0 {
		if cp.EndDate.Before(now) {
			return errors.New("优惠券已失效")
		}
	}

	for _, uid := range uids {
		// 获取客户信息
		cust, err := s.custDao.GetCustomerByID(uid)
		if err != nil {
			utils.LogError("查询客户信息失败: " + err.Error())
			return errors.New("校验客户信息失败")
		}

		if err := s.GiveCouponTo(&user, cust, cp, models.RECEIVEING_TYPE_GIVE); err != nil {
			return err
		}
	}

	return nil
}

// GiveCouponTo 赠送优惠券
// 本方法会被多出调用, 方法内使用 Context 将是不安全的
func (s *CouponServ) GiveCouponTo(from *model.SysUser, to *model.TaCustomer, cp *model.TaCoupon, receivingType string) error {
	if from.UserId == "" {
		return errors.New("必须指定赠送人")
	}

	if to.CustomerId == "" {
		return errors.New("必须指定接收人")
	}

	// 优惠券必须状态正常
	if cp.CouponId == "" || cp.Status != models.STATUS_NORMAL {
		return errors.New("优惠券已失效或者不存在")
	}

	now := time.Now().Local()
	if cp.ValidDays <= 0 {
		if cp.EndDate.Before(now) {
			return errors.New("优惠券已失效")
		}
	}

	if cp.TotalCount < cp.SentCount+1 {
		return errors.New("券已经没有库存！")
	}

	// 计算卡券有效期
	startDate := cp.StartDate
	endDate := cp.EndDate
	if cp.ValidDays > 0 {
		startDate = now
		hs := int64(cp.ValidDays * 24)
		endDate = now.Add(time.Hour * time.Duration(hs))
	}

	// 检查用户是否已经有了该券
	// custCPS, err := s.dao.CheckCustCoupon(to.CustomerId, cp.CouponId, startDate, endDate)
	// if err != nil {
	// 	utils.LogError("校验客户优惠券失败: " + err.Error())
	// 	return errors.New("校验客户优惠券失败")
	// }
	// beego.Error(custCPS)
	// if custCPS != nil && len(custCPS) > 0 {
	// 	return errors.New("客户 " + to.CustomerName + " 已经拥有该券")
	// }

	// 放入我的卡券
	custCP := model.TaCustomerCoupon{
		CouponId:           cp.CouponId,
		CustomerCouponName: cp.CouponName,
		CustomerId:         to.CustomerId,
		StartDate:          startDate,
		EndDate:            endDate,
		ReceiveDate:        now,
		CaseId:             cp.CaseId,
		OrgId:              cp.OrgId,
		ReceivingType:      receivingType,
		Price:              cp.Price,
	}

	// 放入赠送记录
	rec := model.TaCouponGiveRecord{
		GiftType:   models.GIVE_GIFT_COUPON,
		GiftId:     cp.CouponId,
		GiftName:   cp.CouponName,
		FromId:     from.UserId,
		FromName:   from.RealName,
		ToId:       to.CustomerId,
		ToName:     to.CustomerName,
		CreateDate: now,
	}

	// 入库
	if err := s.dao.SaveCustomerCoupons([]model.TaCustomerCoupon{custCP}); err != nil {
		utils.LogError("保存客户优惠券失败: " + err.Error())
		return errors.New("保存客户优惠券失败")
	}

	if err := s.dao.SaveCouponRecoreds([]model.TaCouponGiveRecord{rec}); err != nil {
		utils.LogError("保存优惠券赠送记录失败: " + err.Error())
		return errors.New("保存优惠券赠送记录失败")
	}

	// TODO
	// 券库存处理
	cp.SentCount = cp.SentCount + 1
	if err := s.dao.UpdateCoupon(cp, []string{"sent_count"}); err != nil {
		utils.LogError("更新优惠券库存失败: " + err.Error())
		return errors.New("更新优惠券库存失败")
	}
	return nil
}

// GetCouponByID 获取优惠券
// 未进行权限校验, 非安全的
func (s *CouponServ) GetCouponByID(cpID string) (*model.TaCoupon, error) {
	cp, err := s.dao.GetCouponByID(cpID)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("校验优惠券失败")
	}

	return cp, nil
}

// GetCouponInfoByID 获取详情
func (s *CouponServ) GetCouponInfoByID(cpID string) (*coupon.CouponInfo, error) {
	cp, err := s.dao.GetCouponInfoByID(cpID)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("校验优惠券失败")
	}

	return cp, nil
}

// GetCouponByIDWithCheck 获取优惠券
func (s *CouponServ) GetCouponByIDWithCheck(cpID string) (*coupon.CouponInfo, error) {
	cp, err := s.dao.GetCouponInfoByID(cpID)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("校验优惠券失败")
	}

	if err := utils.NewAuthEngine(s.ctx).CheckCase(cp.CaseId); err != nil {
		return nil, err
	}

	return cp, nil
}

// GetCouponBySendType 根据发送类型获取优惠券
func (s *CouponServ) GetCouponBySendType(sendtype string) ([]model.TaCoupon, error) {
	cases := s.ctx.Get("cases").([]model.SysUserCase)
	caseIDs := ""
	for _, value := range cases {
		if caseIDs != "" {
			caseIDs = caseIDs + ","
		}
		caseIDs = caseIDs + value.CaseId
	}
	coupon, err := s.dao.GetCouponBySendType(caseIDs, sendtype)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("获取优惠券失败")
	}
	return coupon, nil
}

// GetCouponWithCustomer 获取优惠券明细
func (s *CouponServ) GetCouponWithCustomer(couponid string) (*coupon.CaseCouponDetailWithRealName, error) {
	if couponid == "" {
		return nil, errors.New("没有券信息！")
	}
	couponDetail, err := s.dao.GetCouponDetail(couponid)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("获取优惠券失败")
	}
	return couponDetail, nil
}

// GetCouponByCustomer 获取我的优惠券
func (s *CouponServ) GetCouponByCustomer() ([]coupon.CustomerCoupon, error) {
	org := s.ctx.Get("org").(model.SysOrg)
	customer := s.ctx.Get("customer").(model.TaCustomer)
	coupons, err := s.dao.GetCouponByCustomer(org.OrgId, customer.CustomerId)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("获取优惠券失败")
	}
	return coupons, nil
}

// GetCustomerCouponByID 获取我的优惠券详情
func (s *CouponServ) GetCustomerCouponByID(id string) (*coupon.CustomerCouponWithShare, error) {
	coupon, err := s.dao.GetCustomerCouponByID(id)
	if err != nil {
		utils.LogError("获取优惠券失败: " + err.Error())
		return nil, errors.New("获取优惠券失败")
	}
	return coupon, nil
}

// GetCaseUsableCoupon 获取案场可用券
func (s *CouponServ) GetCaseUsableCoupon(page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	customer := s.ctx.Get("customer").(model.TaCustomer)
	if customer.UserId == "" {
		return nil, errors.New("您没有权限查看数据")
	}
	userTypes, err := s.userDao.GetUserWithTypeByID(customer.UserId)
	if err != nil {
		utils.LogError("获取用户信息失败: " + err.Error())
		return nil, errors.New("获取用户信息失败")
	}
	if len(userTypes) == 0 {
		return nil, errors.New("您没有权限查看数据")
	}
	caseinfo, err := s.userDao.GetUserBelongCase(customer.UserId)
	if err != nil {
		return nil, errors.New("您没有案场信息！")
	}
	var userid = ""
	if userTypes[0].TypeId != models.USERTYPE_MANAGER {
		userid = customer.UserId
	}
	coupons, err := s.dao.GetCaseUsableCoupon(caseinfo.CaseId, userid, page, pageSize)
	if err != nil {
		return nil, errors.New("获取数据失败！")
	}
	total, err := s.dao.GetCaseUsableCouponCount(caseinfo.CaseId, userid)
	if err != nil {
		utils.LogError("获取商品规格失败: " + err.Error())
		return nil, errors.New("获取商品规格失败")
	}
	return map[string]interface{}{
		"list":     coupons,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

// ReceiveCoupon 优惠券领取
func (s *CouponServ) ReceiveCoupon(id, saleid, serialcode string) error {
	if saleid == "" || serialcode == "" {
		return errors.New("log-error-参数不全！不允许领取")
	}

	// 数据校验
	customer := s.ctx.Get("customer").(model.TaCustomer)
	if customer.Phone == "" {
		return errors.New("log-error-用户未绑定手机号！")
	}
	if customer.UserId != "" {
		return errors.New("log-error-不允许内部人员领取优惠券！")
	}

	coupon, err := s.dao.GetCouponByID(id)
	if err != nil {
		utils.LogError("领取优惠券获取优惠券信息失败：", err)
		return errors.New("log-error-领取优惠券失败！")
	}
	if coupon.TotalCount <= coupon.SentCount && coupon.IsOver == 0 {
		return errors.New("log-error-优惠券已领完！")
	}
	if coupon.ValidDays <= 0 && coupon.EndDate.Before(time.Now()) {
		return errors.New("log-error-优惠券已过期")
	}

	// 销售校验
	saler, err := s.custDao.GetUserByID(saleid)
	if err != nil {
		utils.LogError("领取优惠券获取销售信息失败：", err)
		return errors.New("log-error-领取优惠券失败！")
	}
	if saleid != beego.AppConfig.String("defaultShareUserID") {
		caseinfo, err := s.userDao.GetUserBelongCase(saleid)
		if err != nil {
			utils.LogError("领取优惠券获取销售案场信息失败：", err)
			return errors.New("log-error-领取优惠券失败！")
		}
		if caseinfo.CaseId != coupon.CaseId {
			return errors.New("log-error-销售不合法！")
		}
	}

	// 验证码判断
	isok, err := s.dao.CheckBySerialCode(coupon.CouponId, serialcode)
	if err != nil {
		utils.LogError("获取优惠券批次码判断失败：", err)
		return errors.New("log-error-领取优惠券失败！")
	}
	if !isok {
		return errors.New("log-error-该优惠券已被领取！")
	}

	// 计算卡券有效期
	startDate := coupon.StartDate
	endDate := coupon.EndDate
	if coupon.ValidDays > 0 {
		startDate = time.Now().Local()
		hs := int64(coupon.ValidDays * 24)
		endDate = time.Now().Local().Add(time.Hour * time.Duration(hs))
	}

	// 领取
	var customerCoupon = model.TaCustomerCoupon{
		CouponId:           coupon.CouponId,
		CustomerCouponName: coupon.CouponName,
		CustomerId:         customer.CustomerId,
		SalesId:            saler.UserId,
		SalesName:          saler.UserName,
		StartDate:          startDate,
		EndDate:            endDate,
		CaseId:             coupon.CaseId,
		OrgId:              coupon.OrgId,
		SerialCode:         serialcode,
		ReceivingType:      models.RECEIVEING_TYPE_SALES,
		Price:              coupon.Price,
	}

	err = s.dao.SaveCustomerCoupon(&customerCoupon)
	if err != nil {
		utils.LogError("保存用户体验优惠券信息失败：", err)
		return errors.New("log-error-领取优惠券失败！")
	}
	// 回填体验优惠券信息
	coupon.SentCount = coupon.SentCount + 1
	cols := []string{
		"sent_count",
	}
	err = s.dao.UpdateCoupon(coupon, cols)
	if err != nil {
		utils.LogError("回填体验优惠券信息失败:", err)
		return errors.New("log-error-领取优惠券失败！")
	}
	return nil
}

// ChannelReceiveCoupon 渠道端优惠券领取
func (s *CouponServ) ChannelReceiveCoupon(id string) error {
	// 数据校验
	customer := s.ctx.Get("customer").(model.TaCustomer)
	if customer.Phone == "" {
		return errors.New("log-error-用户未绑定手机号！")
	}
	if customer.UserId != "" {
		return errors.New("log-error-不允许内部人员领取优惠券！")
	}

	// 判断是否已领取
	isok, err := s.dao.CheckCustomerHaveCoupon(id, customer.CustomerId)
	if err != nil {
		utils.LogError("判断用户是都已领取失败：", err)
		return errors.New("log-error-领取优惠券失败！")
	}
	if !isok {
		return errors.New("log-error-您已领取过该优惠券！")
	}

	coupon, err := s.dao.GetCouponByID(id)
	if err != nil {
		utils.LogError("领取优惠券获取优惠券信息失败：", err)
		return errors.New("log-error-领取优惠券失败！")
	}
	if coupon.TotalCount <= coupon.SentCount && coupon.IsOver == 0 {
		return errors.New("log-error-优惠券已领完！")
	}

	if coupon.ValidDays <= 0 && coupon.EndDate.Before(time.Now()) {
		return errors.New("log-error-优惠券已过期")
	}

	// 计算卡券有效期
	startDate := coupon.StartDate
	endDate := coupon.EndDate
	if coupon.ValidDays > 0 {
		startDate = time.Now().Local()
		hs := int64(coupon.ValidDays * 24)
		endDate = time.Now().Local().Add(time.Hour * time.Duration(hs))
	}

	// 领取
	var customerCoupon = model.TaCustomerCoupon{
		CouponId:           coupon.CouponId,
		CustomerCouponName: coupon.CouponName,
		CustomerId:         customer.CustomerId,
		StartDate:          startDate,
		EndDate:            endDate,
		CaseId:             coupon.CaseId,
		OrgId:              coupon.OrgId,
		ReceivingType:      models.RECEIVEING_TYPE_CHANNEL,
		Price:              coupon.Price,
	}

	err = s.dao.SaveCustomerCoupon(&customerCoupon)
	if err != nil {
		utils.LogError("保存用户体验优惠券信息失败：", err)
		return errors.New("log-error-领取优惠券失败！")
	}
	// 回填体验优惠券信息
	coupon.SentCount = coupon.SentCount + 1
	cols := []string{
		"sent_count",
	}
	err = s.dao.UpdateCoupon(coupon, cols)
	if err != nil {
		utils.LogError("回填体验优惠券信息失败:", err)
		return errors.New("log-error-领取优惠券失败！")
	}
	return nil
}

// GetCustomerCouponByCustomerAndSale 根据用户与销售信息获取领取记录
func (s *CouponServ) GetCustomerCouponByCustomerAndSale(customerid string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	saler := s.ctx.Get("customer").(model.TaCustomer)
	if saler.UserId == "" {
		return nil, errors.New("您没有权限查看数据")
	}
	coupons, err := s.dao.GetCustomerCouponByCustomerAndSale(customerid, saler.UserId, page, pageSize)
	if err != nil {
		return nil, err
	}
	total, err := s.dao.GetCustomerCouponByCustomerAndSaleCount(customerid, saler.UserId)
	if err != nil {
		utils.LogError("获取用户的领取记录失败: " + err.Error())
		return nil, errors.New("获取用户的领取记录失败")
	}
	return map[string]interface{}{
		"list":     coupons,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

func (s *CouponServ) GetSysCouponByCase(caseid string) ([]model.TaCoupon, error) {
	coupon, err := s.dao.GetSysCouponByCase(caseid)
	if err != nil {
		utils.LogError("获取系统赠送券失败: " + err.Error())
		return nil, errors.New("获取系统赠送券失败")
	}
	return coupon, nil
}
