package course

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/course"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"time"

	"github.com/yl10/kit/guid"
)

// PreOrders 预下单
// 保存订单信息
func (s *CourseServ) PreOrders(
	couseOrder *model.TaCourseOrders,
) (*model.TaCourseOrders, error) {
	org := s.ctx.Get("org").(model.SysOrg)
	cust := s.ctx.Get("customer").(model.TaCustomer)
	// 校验人员
	couseOrder.CustomerId = cust.CustomerId
	if cust.UserId != "" {
		// 内部人员
		// 如果是内部人员购买或者兑换课程, 卡券或者城币就从内部人员自己的账户走
		couseOrder.PayType = models.CONSUME_INNER
	} else {
		// 默认是卡券支付
		couseOrder.PayType = models.CONSUME_COUPON
	}
	// 校验课程
	course, err := s.validCourse(couseOrder)
	if err != nil {
		return nil, err
	}
	if course.CourseDetail == nil || len(course.CourseDetail) == 0 {
		return nil, errors.New("定单课程尚未排课, 不能下单")
	}
	couseOrder.OrdersId = guid.NewGUIDString()
	couseOrder.Price = course.Price // 课程价格, 即为订单价格
	couseOrder.ActualAmount = course.Price
	couseOrder.CouponAmount = "0.0"
	couseOrder.CourseName = course.CourseName
	couseOrder.CourseNum = course.CourseNum
	couseOrder.OrgId = org.OrgId
	couseOrder.Status = models.STATUS_READY
	couseOrder.IsPay = models.BOOL_FALSE

	if cust.Name != "" {
		couseOrder.CustomerName = cust.Name
	} else {
		couseOrder.CustomerName = cust.CustomerName
	}
	// 入库
	neworder, err := s.AddOrder(couseOrder)
	if err != nil {
		utils.LogError("课程下单失败: " + err.Error())
		return nil, errors.New("下单失败, 请重试")
	}
	return neworder, nil
}

// ConfirmOrders 确认订单
func (s *CourseServ) ConfirmOrders(ordersID, customercouponid string) error {

	couseOrder, err := s.dao.GetOrderByID(ordersID)
	if err != nil {
		return utils.LogError("获取订单失败", err)
	}

	if couseOrder.Status != models.STATUS_READY {
		return errors.New("没有订单信息")
	}

	if couseOrder.IsPay == models.BOOL_TRUE {
		return errors.New("订单已付款，请勿重复付款！")
	}

	cust := s.ctx.Get("customer").(model.TaCustomer)

	// TODO
	// 校验案场
	// caseID := couseOrder.CaseId
	// if err := utils.NewAuthEngine(s.ctx).CheckCase(caseID); err != nil {
	// 	return err
	// }

	// 校验课程
	course, err := s.validCourse(couseOrder)
	if err != nil {
		return err
	}
	if course.CourseDetail == nil || len(course.CourseDetail) == 0 {
		return errors.New("定单课程尚未排课, 不能下单")
	}

	account, err := s.custDAO.GetAccountByCust(cust.CustomerId)
	if err != nil {
		utils.LogError("查询用户账户信息出错: " + err.Error())
		return errors.New("查询用户账户信息出错")
	}
	accMoney, _ := strconv.ParseFloat(account.Amount, 64)

	if customercouponid != "" {
		// 校验卡券, 同时进行卡券的注销或者已使用的更新操作
		// TODO
		couseOrder.PayType = models.CONSUME_COUPON

		// 优惠券校验
		customerCoupon, err := s.couponDAO.GetCustomerCouponByID(customercouponid)
		if err != nil {
			utils.LogError("查询优惠券信息失败: " + err.Error())
			return errors.New("查询优惠券信息失败")
		}
		if customerCoupon == nil || customerCoupon.CustomerCouponId == "" {
			return errors.New("优惠券无效！")
		}

		if customerCoupon.CaseId != couseOrder.CaseId {
			return errors.New("优惠券对应案场与下单案场不一致，不允许下单！")
		}

		if customerCoupon.Status != models.STATUS_NORMAL || !customerCoupon.UseDate.IsZero() {
			return errors.New("优惠券已被使用！请重新选择优惠券！")
		}

		// 根据id获取优惠券信息
		coupon, err := s.couponDAO.GetCouponInfoByCustomerCouponID(customercouponid)
		if err != nil {
			utils.LogError("查询优惠券信息失败: " + err.Error())
			return errors.New("查询优惠券信息失败")
		}

		if coupon.StartDate.After(time.Now().Local()) {
			return errors.New("该优惠券还未到有效期，不允许下单！")
		}
		if coupon.EndDate.Before(time.Now().Local()) {
			return errors.New("优惠券已失效，不允许下单！")
		}

		// 判断优惠券是否可以抵用商品
		var isdy = false
		if coupon.IsAll == 1 {
			isdy = true
			couseOrder.CouponAmount = coupon.Price
		} else {
			for _, target := range coupon.Targets {
				if target.TargetId == course.CourseId {
					isdy = true
				}
			}
			couseOrder.CouponAmount = coupon.Price
		}
		if !isdy {
			return errors.New("优惠券不可抵用该课程")
		}
		var ordersCoupon = model.TaCourseOrdersCoupon{
			OrdersId:         couseOrder.OrdersId,
			CouponId:         coupon.CouponId,
			CouponName:       coupon.CouponName,
			UsedAmount:       couseOrder.CouponAmount,
			CustomerCouponId: customerCoupon.CustomerCouponId,
			CourseId:         course.CourseId,
			CourseName:       course.CourseName,
		}
		if err := s.dao.SaveOrdersCoupon(&ordersCoupon, couseOrder); err != nil {
			utils.LogError("保存优惠信息出错: " + err.Error())
			return errors.New("保存优惠信息出错")
		}

		// 优惠券核销
		err = s.couponDAO.VerifyCustomerCoupon(customercouponid)
		if err != nil {
			utils.LogError("优惠券核销出错: " + err.Error())
			return errors.New("优惠券核销出错")
		}
	} else {
		couseOrder.PayType = models.CONSUME_COINCHG
		couseOrder.CouponAmount = "0.0"
	}

	couponAmount, _ := strconv.ParseFloat(couseOrder.CouponAmount, 64)
	payMoney, _ := strconv.ParseFloat(couseOrder.Price, 64)
	payMoney = payMoney - couponAmount

	couseOrder.ActualAmount = strconv.FormatFloat(payMoney, 'f', -1, 64)

	// 校验金额
	// if err := s.validBillCharges(couseOrder, coupons); err != nil {
	// 	return err
	// }

	// 实际支付
	actualAmount := payMoney

	// 用户账户  -- 内部人员也可以购买
	if actualAmount > 0 {
		var leftPay = actualAmount
		vips, err := s.custDAO.GetValidVIPCards(cust.CustomerId)
		if err != nil {
			return utils.LogError("查询用户 VIP卡 出错", err)
		}

		var vipAccount float64
		if vips != nil && len(vips) > 0 {
			for _, vip := range vips {
				vipMoney, _ := strconv.ParseFloat(vip.Balance, 64)
				vipAccount += vipMoney

				// 当前扣款
				var vipPay float64
				if vipMoney > leftPay {
					// 如果卡余额比 需要扣款的多, 那么就只扣这一张卡
					// 通过 leftPay < 0.01 来判断
					vipPay = leftPay
					leftPay = 0
				} else {
					// 如果余额不够, 先扣掉本张卡余额, 继续循环
					vipPay = vipMoney
					leftPay -= vipMoney
				}

				if err := s.saveVipChangeRec(vipPay, &vip, couseOrder); err != nil {
					return utils.LogError("VIP卡扣款失败", err.Error())
				}

				if leftPay < 0.01 {
					break
				}
			}
		}

		// 如果 VIP 卡余额不足, 则使用账户余额
		if leftPay > 0.01 {
			if leftPay > accMoney {
				return errors.New("账户余额不足")
			}

			// 插入账户流水
			if err := s.saveCustomerPayRec(account, couseOrder, leftPay); err != nil {
				utils.LogError("插入账户流水出错: " + err.Error())
				return errors.New("写入账户流水出错")
			}
		}
	}

	// 更新订单信息
	if err := s.SaveOrder(couseOrder, course); err != nil {
		utils.LogError("课程下单失败: " + err.Error())
		return errors.New("下单失败, 请重试")
	}

	// 发送短信
	utils.SendSMS("orders", cust.Phone, course.CourseName)

	return nil
}

// saveVipChangeRec 保存vip消费记录
func (s *CourseServ) saveVipChangeRec(payMoney float64, custVIP *model.TaCustomerVip, info *model.TaCourseOrders) error {
	vipChange := new(model.TaCustomerVipChange)
	vipChange.CustomerVipId = custVIP.CustomerVipId
	vipChange.CustomerId = info.CustomerId
	vipChange.OrgId = info.OrgId
	vipChange.CaseId = info.CaseId
	vipChange.ChangeType = models.CONSUME_COINCHG
	vipChange.FloatType = models.ACCOUNT_SPENT
	vipChange.SourceId = info.OrdersId
	vipChange.SourceName = info.CustomerName + "课程下单"
	vipChange.Amount = strconv.FormatFloat(payMoney, 'f', -1, 64)

	if err := s.vipDAO.InsertCustomerVipChange(*vipChange); err != nil {
		return err
	}

	return s.custDAO.UpdateCustVipCardAmount(custVIP, payMoney)
}

// validCourse 校验课程
func (s *CourseServ) validCourse(order *model.TaCourseOrders) (*course.CourseDetail, error) {
	if order.CourseId == "" {
		return nil, errors.New("没有获取到下单课程")
	}

	// 课程
	course, err := s.dao.GetCourseInfo(order.CourseId)
	if err != nil {
		utils.LogError("获取课程信息失败: " + err.Error())
		return nil, errors.New("获取课程信息失败")
	}

	if course.Status != models.STATUS_NORMAL {
		return nil, errors.New("课程状态不正确, 不能购买或者兑换")
	}

	return course, nil
}

// validCustomer 校验客户
func (s *CourseServ) validCustomer(custID string) (*model.TaCustomer, error) {
	if custID == "" {
		return nil, errors.New("没有获取到下单人员")
	}

	cust, err := s.custDAO.GetCustomerByID(custID)
	if err != nil {
		utils.LogError("查询下单人出错: " + err.Error())
		return nil, errors.New("验证下单人出错")
	}

	if cust.Status != models.STATUS_NORMAL {
		return nil, errors.New("下单人状态不正确, 不能下单")
	}

	return cust, nil
}

// validBillCharges 校验各种金额, 数量等
func (s *CourseServ) validBillCharges(
	couseOrder *model.TaCourseOrders,
	coupons []model.TaCourseOrdersCoupon,
) error {
	orderCost, _ := strconv.ParseFloat(couseOrder.Price, 64)
	// 算法开始, 实际支付 == 订单金额
	actualAmount := orderCost
	// 优惠金额是 0
	couponAmount := float64(0.0)

	couseOrder.ActualAmount = strconv.FormatFloat(actualAmount, 'f', -1, 64)
	couseOrder.CouponAmount = strconv.FormatFloat(couponAmount, 'f', -1, 64)

	// 如果不是使用的卡券
	if coupons == nil || len(coupons) == 0 {
		return nil
	}

	// 目前的业务是, 购买课程只能使用一张卡券
	// 这里使用的是数组, 相当于做一个拓展
	for _, coupon := range coupons {
		// TODO 从卡券字典中获取价格
		// 目前是前端传过来的
		dedCharge, err := strconv.ParseFloat(coupon.UsedAmount, 64)
		if err != nil {
			utils.LogError("获取卡券抵用金额失败: " + err.Error())
			return errors.New("获取卡券抵用金额错误")
		}

		if dedCharge < 0 {
			return errors.New("卡券抵用金额不能为负值")
		}

		couponAmount += dedCharge
	}

	// 如果优惠券金额 > 订单金额
	if couponAmount > orderCost {
		couponAmount = orderCost
		actualAmount = float64(0)
	} else {
		actualAmount = orderCost - couponAmount
	}

	couseOrder.ActualAmount = strconv.FormatFloat(actualAmount, 'f', -1, 64)
	couseOrder.CouponAmount = strconv.FormatFloat(couponAmount, 'f', -1, 64)
	return nil
}

// saveCustomerPayRec 保存账户消费流水
func (s *CourseServ) saveCustomerPayRec(account *model.TaCustomerAccount, info *model.TaCourseOrders, leftPay float64) error {
	accBill := new(model.TaAccountChange)
	accBill.AccountId = account.AccountId
	accBill.Amount = strconv.FormatFloat(leftPay, 'f', -1, 64)
	accBill.CaseId = info.CaseId
	accBill.ChangeSource = models.ACCSOURCE_COURSE_ORDERS
	accBill.ChangeType = models.CONSUME_COINCHG
	accBill.CustomerId = info.CustomerId
	accBill.CustomerName = info.CustomerName
	accBill.FloatType = models.ACCOUNT_SPENT
	accBill.SourceId = info.OrdersId
	accBill.SourceName = ""

	return s.custDAO.InsertAccountRecords(accBill)
}

// AddOrder 保存订单信息
func (s *CourseServ) AddOrder(order *model.TaCourseOrders) (*model.TaCourseOrders, error) {
	// 订单信息
	order.Status = models.STATUS_READY
	neworder, err := s.dao.SaveCourseOrder(order)
	if err != nil {
		utils.LogError("课程下单失败: " + err.Error())
		return nil, errors.New("下单失败, 请重试")
	}
	return neworder, nil
}

// SaveOrder 保存订单明细
func (s *CourseServ) SaveOrder(order *model.TaCourseOrders, course *course.CourseDetail) error {
	// 订单信息
	if err := s.dao.UpdateOrderPay(order); err != nil {
		utils.LogError("课程下单失败: " + err.Error())
		return errors.New("下单失败, 请重试")
	}

	// 我的课程信息
	custCourse := model.TaCustomerCourse{
		CourseId:         order.CourseId,
		CustomerId:       order.CustomerId,
		OrgId:            order.OrgId,
		CaseId:           order.CaseId,
		CourseName:       course.CourseName,
		LocationId:       course.LocationId,
		Price:            course.Price,
		CourseNum:        course.CourseNum,
		JoinNum:          0,
		CreateDate:       time.Now().Local(),
		CourseObtaimType: order.PayType,
		SourceId:         order.OrdersId,
		IsDone:           models.BOOL_FALSE,
	}

	details := make([]model.TaCourseOrdersDetail, 0)
	custCourseDetails := make([]model.TaCustomerCourseDetail, 0)

	for _, subCourse := range course.CourseDetail {
		d1 := model.TaCourseOrdersDetail{
			OrdersDetailId: guid.NewGUIDString(),
			OrdersId:       order.OrdersId,
			CourseId:       course.CourseId,
			CourseDetailId: subCourse.DetailId,
			BeginDate:      subCourse.BeginDate,
			EndDate:        subCourse.EndDate,
		}
		details = append(details, d1)

		d2 := model.TaCustomerCourseDetail{
			CourseId:     course.CourseId,
			CourseName:   course.CourseName,
			CustomerId:   order.CustomerId,
			DetailId:     subCourse.DetailId,
			StartDate:    subCourse.BeginDate,
			EndDate:      subCourse.EndDate,
			VerifyStatus: models.VERIFY_USEABLE,
			CaseId:       order.CaseId,
			OrgId:        order.OrgId,
		}
		custCourseDetails = append(custCourseDetails, d2)
	}
	customerCourseId, err := s.dao.SaveCourseOfCustomer(&custCourse, custCourseDetails)
	if err != nil {
		utils.LogError("插入我的课程失败: " + err.Error())
		return errors.New("写入我的课程失败")
	}
	custQRCode := model.TaCustomerCourseQrcode{
		CustomerCourseId: customerCourseId,
		CustomerQrcode:   utils.GenerateQRCode(),
		CreateDate:       time.Now(),
		Status:           models.STATUS_NORMAL,
		QrcodeType:       models.QRCODE_TYPE_COURSE,
	}

	if err := s.dao.SaveCourseOrderDetail(details); err != nil {
		utils.LogError("插入课程订单详情: " + err.Error())
		return errors.New("写入课程订单详情失败")
	}
	if err := s.dao.SaveCustomerCourseQrcode(&custQRCode); err != nil {
		utils.LogError("插入我的课程二维码失败: " + err.Error())
		return errors.New("写入我的课程二维码失败")
	}

	return nil
}
