package goods

import (
	"errors"
	"math"
	"spaceofcheng/services/models"
	casemodel "spaceofcheng/services/models/cases"
	"spaceofcheng/services/models/goods"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service"
	"spaceofcheng/services/service/cases"
	"spaceofcheng/services/utils"
	"strconv"
	"time"

	"github.com/yl10/kit/guid"
)

// PreOrders 预下单
func (s *GoodsServ) PreOrders(
	info *model.TaGoodsOrders,
	details []model.TaGoodsOrdersDetail,
	innerUser string,
) (*model.TaGoodsOrders, error) {
	cust := s.ctx.Get("customer").(model.TaCustomer)

	info.OrdersId = guid.NewGUIDString()
	info.MakeStatus = goods.MAKESTATUS_NOTSTARTED
	info.UserId = cust.CustomerId
	info.IsPay = models.BOOL_FALSE
	info.ActualAmount = "0.0"
	info.CouponAmount = "0.0"
	num, err := s.dao.GetOrderNumByUserNow(info.UserId)
	if err != nil {
		utils.LogError("获取5秒钟内是否下单错误:", err)
		return nil, errors.New("获取5秒钟内是否下单错误")
	}
	if num > 0 {
		return nil, errors.New("5 秒内不能频繁下单")
	}

	// 校验下单内容
	if err := s.validOrdersInfo(info); err != nil {
		return nil, err
	}

	// 校验案场信息
	if err := s.validOrdersCase(info, cust); err != nil {
		return nil, err
	}

	// 校验各种金额
	if err := s.validBillCharges(info, details); err != nil {
		return nil, err
	}

	if innerUser == "sales" {
		info.PayType = models.CONSUME_INNER
		// 内部人员, 可以直接购买
		// TODO
	} else {
		info.PayType = models.CONSUME_COUPON
	}

	// 保存主订单
	if err := s.dao.SaveOrders(info); err != nil {
		return nil, err
	}

	// 保存订单明细
	if err := s.dao.SaveOrdersDetail(details, info.OrdersId); err != nil {
		return nil, err
	}

	return info, nil
}

// ConfirmOrder 订单确认
func (s *GoodsServ) ConfirmOrder(ordersID, customercouponid, remark string) error {
	info, err := s.dao.GetOrdersByID(ordersID)
	if err != nil {
		return utils.LogError("获取订单失败", err)
	}

	if info.Status != models.STATUS_READY {
		return errors.New("没有订单信息")
	}

	if info.IsPay == models.BOOL_TRUE {
		return errors.New("订单已付款，请勿重复付款！")
	}

	if info.Remark == "" {
		info.Remark = remark
	}

	if info.PayType == "sales" {
		// 内部人员, 可以直接购买
		// TODO
	} else {
		// 普通客户
		account, err := s.custDAO.GetAccountByCust(info.UserId)
		if err != nil {
			utils.LogError("查询用户账户信息出错: " + err.Error())
			return errors.New("查询用户账户信息出错")
		}

		// 如果是使用优惠券
		if customercouponid != "" {
			info.PayType = models.CONSUME_COUPON

			// 优惠券校验
			customerCoupon, err := s.couponDAO.GetCustomerCouponByID(customercouponid)
			if err != nil {
				utils.LogError("查询优惠券信息失败: " + err.Error())
				return errors.New("查询优惠券信息失败")
			}
			if customerCoupon == nil || customerCoupon.CustomerCouponId == "" {
				return errors.New("优惠券无效！")
			}

			if customerCoupon.CaseId != info.CaseId {
				return errors.New("优惠券对应案场与下单案场不一致，不允许下单！")
			}

			if customerCoupon.Status != models.STATUS_NORMAL || !customerCoupon.UseDate.IsZero() {
				return errors.New("优惠券已被使用！请重新选择优惠券！")
			}

			if customerCoupon.StartDate.After(time.Now().Local()) {
				return errors.New("该优惠券还未到有效期，不允许下单！")
			}
			if customerCoupon.EndDate.Before(time.Now().Local()) {
				return errors.New("优惠券已失效，不允许下单！")
			}
			// 根据id获取优惠券信息
			coupon, err := s.couponDAO.GetCouponInfoByCustomerCouponID(customercouponid)
			if err != nil {
				utils.LogError("查询优惠券信息失败: " + err.Error())
				return errors.New("查询优惠券信息失败")
			}
			// 遍历优惠券可抵用的商品
			var dyGoods = model.TaGoodsOrdersDetail{
				Price:   "0.0",
				GoodsId: "",
			}
			for _, detail := range info.Goods {
				if coupon.IsAll == 1 {
					dyprice, _ := strconv.ParseFloat(dyGoods.Price, 64)
					detailprice, _ := strconv.ParseFloat(detail.Price, 64)
					if dyprice < detailprice {
						dyGoods = detail.TaGoodsOrdersDetail
					}
				} else {
					for _, target := range coupon.Targets {
						if target.TargetId == detail.GoodsId {
							dyprice, _ := strconv.ParseFloat(dyGoods.Price, 64)
							detailprice, _ := strconv.ParseFloat(detail.Price, 64)
							if dyprice < detailprice {
								dyGoods = detail.TaGoodsOrdersDetail
							}
						}
					}
				}
			}
			if dyGoods.GoodsId != "" {
				// 优惠券可以抵用商品
				var ordersCoupon = model.TaGoodsOrdersCoupon{
					OrdersId:         info.OrdersId,
					OrdersNo:         info.OrdersNo,
					CouponId:         coupon.CouponId,
					CouponName:       coupon.CouponName,
					OrgId:            coupon.OrgId,
					GoodsId:          dyGoods.GoodsId,
					GoodsName:        dyGoods.GoodsName,
					UsedAmount:       dyGoods.Price,
					CustomerCouponId: customerCoupon.CustomerCouponId,
				}
				if err := s.dao.SaveOrdersCoupon(&ordersCoupon, &info.TaGoodsOrders); err != nil {
					utils.LogError("保存优惠信息出错: " + err.Error())
					return errors.New("保存优惠信息出错")
				}
				info.CouponAmount = dyGoods.Price
				// info.ActualAmount = "0.0"

				// 优惠券核销
				err := s.couponDAO.VerifyCustomerCoupon(customercouponid)
				if err != nil {
					utils.LogError("优惠券核销出错: " + err.Error())
					return errors.New("优惠券核销出错")
				}
			} else {
				return errors.New("优惠券不可抵用商品")
			}
		} else {
			// 如果是使用城币
			info.PayType = models.CONSUME_COINCHG
			info.CouponAmount = "0.0"
		}

		accMoney, _ := strconv.ParseFloat(account.Amount, 64)
		couponAmount, _ := strconv.ParseFloat(info.CouponAmount, 64)
		payMoney, _ := strconv.ParseFloat(info.Amount, 64)
		payMoney = payMoney - couponAmount

		// 如果是城币, 则插入用户账户消费记录
		if payMoney > 0 {
			var leftPay = payMoney
			vips, err := s.custDAO.GetValidVIPCards(info.UserId)
			if err != nil {
				return utils.LogError("查询用户 VIP卡 出错", err)
			}

			var vipAccount float64
			if vips != nil && len(vips) > 0 {
				for _, vip := range vips {
					vipMoney, _ := strconv.ParseFloat(vip.Balance, 64)
					vipAccount += vipMoney

					// 当前扣款
					var vipPay float64
					if vipMoney > leftPay {
						// 如果卡余额比 需要扣款的多, 那么就只扣这一张卡
						// 通过 leftPay < 0.01 来判断
						vipPay = leftPay
						leftPay = 0
					} else {
						// 如果余额不够, 先扣掉本张卡余额, 继续循环
						vipPay = vipMoney
						leftPay -= vipMoney
					}

					if err := s.saveVipChangeRec(vipPay, &vip, &info.TaGoodsOrders); err != nil {
						return utils.LogError("VIP卡扣款失败", err.Error())
					}

					if leftPay < 0.01 {
						break
					}
				}
			}
			// 如果 VIP 卡余额不足, 则使用账户余额
			if leftPay > 0.01 {
				if leftPay > accMoney {
					return errors.New("账户余额不足")
				}

				// 插入账户流水
				if err := s.saveCustomerPayRec(leftPay, account, &info.TaGoodsOrders); err != nil {
					utils.LogError("插入账户流水出错: " + err.Error())
					return errors.New("写入账户流水出错")
				}
			}
		}
	}

	// 更新订单
	orders := info.TaGoodsOrders
	orders.IsPay = models.BOOL_TRUE
	orders.Status = models.STATUS_NORMAL

	if orders.ActualAmount == "" {
		orders.ActualAmount = "0.0"
	}

	if orders.CouponAmount == "" {
		orders.CouponAmount = "0.0"
	}

	if err := s.dao.UpdateOrders(&orders, []string{
		"status", "is_pay", "coupon_amount", "actual_amount",
	}); err != nil {
		return nil
	}

	newOrder, err := s.GetNewGoods(info.OrdersId)
	if err != nil {
		// 下单流程已经结束, 不反馈错误
		utils.LogError("获取数据失败：", err.Error())
		return nil
	}

	msg := utils.Message{
		To: utils.ClientID{
			Groups: []string{info.CaseId},
		},
		Data: map[string][]interface{}{
			"refreshOnlineGoodsOrders": []interface{}{*newOrder},
		},
	}

	utils.LogError("调用下单 Websocket ")

	utils.SendMessage(msg, utils.WsMessage)

	return nil
}

// saveCustomerPayRec 保存账户消费流水
func (s *GoodsServ) saveCustomerPayRec(payMoney float64, account *model.TaCustomerAccount, info *model.TaGoodsOrders) error {
	actAmount := strconv.FormatFloat(payMoney, 'f', -1, 64)

	accBill := new(model.TaAccountChange)
	accBill.AccountId = account.AccountId
	accBill.Amount = actAmount
	accBill.CaseId = info.CaseId
	accBill.ChangeSource = models.ACCSOURCE_GOODS_ORDERS
	accBill.ChangeType = models.CONSUME_COINCHG
	accBill.CustomerId = info.UserId
	accBill.CustomerName = info.UserName
	accBill.FloatType = models.ACCOUNT_SPENT
	accBill.SourceId = info.OrdersId
	accBill.SourceName = ""

	return s.custDAO.InsertAccountRecords(accBill)
}

// saveVipChangeRec 保存vip消费记录
func (s *GoodsServ) saveVipChangeRec(payMoney float64, custVIP *model.TaCustomerVip, info *model.TaGoodsOrders) error {
	vipChange := new(model.TaCustomerVipChange)
	vipChange.CustomerVipId = custVIP.CustomerVipId
	vipChange.CustomerId = info.UserId
	vipChange.OrgId = info.OrgId
	vipChange.CaseId = info.CaseId
	vipChange.ChangeType = models.CONSUME_COINCHG
	vipChange.FloatType = models.ACCOUNT_SPENT
	vipChange.SourceId = info.OrdersId
	vipChange.SourceName = info.UserName + " 商品下单"
	vipChange.Amount = strconv.FormatFloat(payMoney, 'f', -1, 64)

	if err := s.vipDAO.InsertCustomerVipChange(*vipChange); err != nil {
		return err
	}

	return s.custDAO.UpdateCustVipCardAmount(custVIP, payMoney)
}

// validOrdersInfo 校验下单内容
// 没有对金额, 数量的关系进行校验
func (s *GoodsServ) validOrdersInfo(info *model.TaGoodsOrders) error {

	caseID := info.CaseId
	if caseID == "" {
		return errors.New("未指定消费案场")
	}

	areaID := info.AreaId
	if areaID == "" {
		return errors.New("未指定消费区域")
	}

	tableID := info.TableId
	if tableID == "" {
		return errors.New("未指定消费桌位")
	}

	custID := info.UserId
	if custID == "" {
		return errors.New("未指定下单人员")
	}

	// 验证区域
	caseArea, err := s.caseDAO.GetCaseAreaByID(areaID)
	if err != nil {
		utils.LogError("查询案场区域出错: " + err.Error())
		return errors.New("验证案场区域出错")
	}

	info.AreaName = caseArea.AreaName
	info.OrgId = caseArea.OrgId

	// 验证桌位
	table, err := s.caseDAO.GetCaseTableByID(tableID)
	if err != nil {
		utils.LogError("查询案场桌位出错: " + err.Error())
		return errors.New("验证案场桌位出错")
	}

	info.TableNo = table.TableNo

	// 验证人员
	cust, err := s.custDAO.GetCustomerByID(custID)
	if err != nil {
		utils.LogError("查询下单人出错: " + err.Error())
		return errors.New("验证下单人出错")
	}

	if cust.Status != models.STATUS_NORMAL {
		return errors.New("下单人状态不正确, 不能下单")
	}

	info.UserType = models.USERTYPE_CUSTOMER

	// 如果是管理人员
	if cust.UserId != "" {
		// 查找用户类型
		userWithTypes, err := s.userDAO.GetUserWithTypeByID(cust.UserId)
		if err != nil {
			utils.LogError("查询下单人类型出错: " + err.Error())
			return errors.New("验证下单人出错")
		}
		if userWithTypes != nil && len(userWithTypes) > 0 {
			for _, ut := range userWithTypes {
				if ut.Status != models.STATUS_NORMAL {
					return errors.New("下单人状态不正确, 不能下单")
				}

				info.UserType = ut.TypeId

				// 下单人姓名 - 姓名不存在则用昵称
				info.UserName = ut.RealName
				if info.UserName == "" {
					info.UserName = cust.CustomerName
				}
			}
		}

		if info.PayType == models.CONSUME_INNER {
			// 查找禁用名单
			fibUsers, err := s.userDAO.GetForbidUserByUserID(cust.UserId)
			if err != nil {
				utils.LogError("查询下单人是否被禁出错: " + err.Error())
				return errors.New("验证下单人出错")
			}

			if fibUsers != nil && len(fibUsers) > 0 {
				for _, u := range fibUsers {
					if u.ForbidType == models.FORBID_ORDER && u.Status == models.STATUS_NORMAL {
						return errors.New("当前下单人被禁止点单")
					}
				}
			}
		}
	}

	return nil
}

// validBillCharges 验证下单各种金额
func (s *GoodsServ) validBillCharges(info *model.TaGoodsOrders, details []model.TaGoodsOrdersDetail) error {
	if details == nil || len(details) == 0 {
		return errors.New("没有发现下单明细")
	}

	caseID := info.CaseId

	goodsAmount := float64(0.0)
	for _, item := range details {

		var price float64
		var goodsPrice float64

		price, _ = strconv.ParseFloat(item.Price, 64)

		// 商品无规格
		if item.SpecId == "" {
			goods, err := s.dao.GetGoodsByID(item.GoodsId)
			if err != nil {
				utils.LogError("查询商品详情失败: " + err.Error())
				return errors.New("校验商品信息失败")
			}

			goodsPrice, _ = strconv.ParseFloat(goods.Price, 64)
		} else {
			goods, err := s.dao.GetGoodsWithPriceByID(item.GoodsId, item.SpecId, caseID)
			if err != nil {
				utils.LogError("查询商品详情失败: " + err.Error())
				return errors.New("校验商品信息失败")
			}

			goodsPrice, _ = strconv.ParseFloat(goods.GoodsPrice, 64)
		}

		if math.Abs(price-goodsPrice) > 0.00001 {
			return errors.New("商品金额计算错误")
		}

		// TODO 校验库存
		// 商品暂时无库存

		goodsAmount += price * float64(item.Number)
	}

	// 校验总金额
	orderAmount, _ := strconv.ParseFloat(info.Amount, 64)

	if math.Abs(orderAmount-goodsAmount) > 0.00001 {
		return errors.New("商品金额计算错误")
	}

	// TODO
	// 暂时实际支付为 0
	info.ActualAmount = "0.0"

	return nil
}

// GetOrderList 获取订单列表
// 管理端 - 查询统计
func (s *GoodsServ) GetOrderList(caseID, userType, status string, orderDate []time.Time, page, pagesize int) ([]goods.OrdersWithGoods, int64, error) {
	if err := utils.NewAuthEngine(s.ctx).CheckCase(caseID); err != nil {
		return nil, 0, err
	}

	var makeStatus []string
	if status != "" {
		makeStatus = []string{status}
	}

	if page < 1 {
		page = 1
	}
	if pagesize < 0 {
		pagesize = 0
	}
	limit := []int{
		pagesize,
		(page - 1) * pagesize,
	}

	orderList, cnt, err := s.dao.GetOrdersByCaseID(caseID, userType, makeStatus, orderDate, limit)
	if err != nil {
		utils.LogError("获取商品订单列表失败： " + err.Error())
		return nil, 0, errors.New("获取商品订单列表失败")
	}

	return orderList, cnt, nil
}

// GetOnLineOrders 获取订单列表
// 管理端 - 在线订单
func (s *GoodsServ) GetOnLineOrders(caseID string, checkRights ...bool) ([]goods.OrdersWithGoods, error) {
	needRights := true
	if checkRights != nil && len(checkRights) > 0 {
		needRights = checkRights[0]
	}

	if needRights {
		if err := utils.NewAuthEngine(s.ctx).CheckCase(caseID); err != nil {
			return nil, err
		}
	}

	makeStatus := []string{goods.MAKESTATUS_NOTSTARTED, goods.MAKESTATUS_DOING}

	orderList, _, err := s.dao.GetOrdersByCaseID(caseID, "", makeStatus, nil, nil)
	if err != nil {
		utils.LogError("获取在线订单列表失败： " + err.Error())
		return nil, errors.New("获取在线订单列表失败")
	}

	return orderList, nil
}

// GetOrdersByRecord 获取月订单
// 管理端
func (s *GoodsServ) GetOrdersByRecord(recordid string) ([]goods.OrdersWithGoods, error) {
	orders, err := s.dao.GetOrdersByRecord(recordid)
	if err != nil {
		utils.LogError("获取订单列表失败： " + err.Error())
		return nil, errors.New("获取订单列表失败")
	}
	return orders, nil
}

// validOrdersCase 校验案场信息
func (s *GoodsServ) validOrdersCase(info *model.TaGoodsOrders, cust model.TaCustomer) error {
	conf, err := s.caseDAO.GetConfByCase(info.CaseId, cases.CONFTYPE_GOODS)
	if err != nil {
		utils.LogError("获取配置信息失败： " + err.Error())
		return errors.New("获取配置信息失败")
	}

	if conf == nil || conf.Status != models.STATUS_NORMAL {
		return errors.New("当前案场已歇业！不允许下单！")
	}

	if cust.UserId != "" {
		// 获取当月记录
		record, err := s.caseDAO.GetRecordByCase(info.CaseId)
		if err != nil {
			utils.LogError("获取月记录失败: " + err.Error())
			return errors.New("获取月记录失败")
		}
		if record != nil && record.IsOver == casemodel.ISNOOVER && record.LeftNum < info.OrdersNum {
			return errors.New("超出案场限制的本月下单次数！不允许下单！")
		}
	}
	return nil
}

// GetCustomerOrders 获取用户订单
func (s *GoodsServ) GetCustomerOrders(page, pageSize int, fromClient string) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	cust := s.ctx.Get("customer").(model.TaCustomer)
	list, err := s.dao.GetCustomerOrders(cust.CustomerId, page, pageSize, fromClient)
	if err != nil {
		utils.LogError("获取订单信息失败: " + err.Error())
		return nil, errors.New("获取订单信息失败")
	}
	total, err := s.dao.GetCustomerOrdersCount(cust.CustomerId, fromClient)
	if err != nil {
		utils.LogError("获取订单信息失败: " + err.Error())
		return nil, errors.New("获取订单信息失败")
	}

	return map[string]interface{}{
		"list":     list,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

// GetCaseOrdersNum 获取案场订单数量
func (s *GoodsServ) GetCaseOrdersNum(caseid string) (map[string]interface{}, error) {
	customer := s.ctx.Get("customer").(model.TaCustomer)
	if customer.UserId == "" {
		return nil, errors.New("您没有权限查看此数据！")
	}
	total, err := s.dao.GetOrdersCountByCase(caseid, customer.CustomerId)
	if err != nil {
		utils.LogError("获取案场订单数量失败: " + err.Error())
		return nil, errors.New("获取案场信息失败")
	}
	monthtotal, err := s.dao.GetMonthOrdersCountByCase(caseid, customer.CustomerId)
	if err != nil {
		utils.LogError("获取月案场订单数量失败: " + err.Error())
		return nil, errors.New("获取案场信息失败")
	}
	return map[string]interface{}{
		"total":      total,
		"monthtotal": monthtotal,
	}, nil
}

// FinishMake 完成制作
func (s *GoodsServ) FinishMake(id string) error {
	var orders = model.TaGoodsOrders{
		OrdersId:   id,
		MakeStatus: goods.MAKESTATUS_FINISHED,
	}
	err := s.dao.UpdateOrdersMake(orders)
	return err
}

// CancelMake 取消制作
func (s *GoodsServ) CancelMake(id string) error {
	var orders = model.TaGoodsOrders{
		OrdersId:   id,
		MakeStatus: goods.MAKESTATUS_CANCEL,
	}
	err := s.dao.UpdateOrdersMake(orders)
	return err
}

// UpdateOrdersIntimidate 更新订单打单
func (s *GoodsServ) UpdateOrdersIntimidate(id string) error {
	err := s.dao.UpdateOrdersIntimidate(id)
	if err != nil {
		utils.LogError("更新失败：", err)
		return err
	}
	return nil
}

// GetNewGoods 获取新增的订单
func (s *GoodsServ) GetNewGoods(ordersID string) (*goods.OrdersWithGoods, error) {
	order, err := s.dao.GetOrdersByID(ordersID)
	if err != nil {
		utils.LogError("获取新增订单列表失败", err)
		return nil, errors.New("获取新增订单列表失败")
	}

	return order, nil
}
