package vipcard

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/customer"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/models/vipcard"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
)

// CaseServ 系统处理
type VipcardServ struct {
	ctx  *utils.Context
	dao  *vipcard.VipcardDAO
	cdao *customer.CustomerDAO
}

// NewVipcardServ 初始化
func NewVipcardServ(ctx *utils.Context) *VipcardServ {
	return &VipcardServ{
		ctx:  ctx,
		dao:  vipcard.NewVipcardDAO(ctx),
		cdao: customer.NewCustomerDAO(ctx),
	}
}

// GetVipChild 获取VIP卡列表
func (s *VipcardServ) GetVipChild(caseids, cardNo, sellerName, userName string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	vipchild, err := s.dao.GetVipCardList(caseids, cardNo, sellerName, userName, page, pageSize)
	if err != nil {
		utils.LogError("获取VIP卡信息失败: " + err.Error())
		return nil, errors.New("获取VIP卡信息失败")
	}
	total, err := s.dao.GetCardCount(caseids, cardNo, sellerName, userName)
	if err != nil {
		utils.LogError("获取VIP卡信息失败: " + err.Error())
		return nil, errors.New("获取VIP卡信息失败")
	}
	return map[string]interface{}{
		"list":     vipchild,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, err
}

// 根据code获取vip子卡
func (s *VipcardServ) GetVipChildByCode(vipChildId string) (*model.TaVipCardChild, error) {
	vipCardChild, err := s.dao.GetVipChildByCode(vipChildId)
	if err != nil {
		utils.LogError("获取VIP卡信息失败: " + err.Error())
		return nil, errors.New("获取VIP卡信息失败")
	}
	return vipCardChild, err
}

// UpdateVipChild 更新子卡
func (s *VipcardServ) UpdateVipChild(vipChild *model.TaVipCardChild) error {
	var err error
	err = s.dao.UpdateVipChild(vipChild)
	if err != nil {
		utils.LogError("获取VIP卡信息失败: " + err.Error())
		return errors.New("获取VIP卡信息失败")
	}
	return err
}

// AddVipCard 新建VIP卡
func (s *VipcardServ) AddVipCard(vipCard model.TaVipCard) (*model.TaVipCard, error) {
	vipcard, err := s.dao.AddVipCard(vipCard)
	if err != nil {
		utils.LogError("创建VIP卡失败: " + err.Error())
		return nil, errors.New("创建VIP卡失败")
	}
	for a := 0; a < vipcard.TotalCount; a++ {
		var vipChild model.TaVipCardChild
		vipChild.VipCardId = vipcard.VipCardId
		vipChild.VipCardChildCode = s.dao.GenerateChildCode()
		vipChild.Amount = vipCard.CardAmount
		vipChild.BeginDate = vipCard.BeginDate
		vipChild.EndDate = vipCard.EndDate
		vipChild.CaseId = vipCard.CaseId
		vipChild.OrgId = vipCard.OrgId
		_, err := s.dao.AddVipChildCard(vipChild)
		if err != nil {
			utils.LogError("创建VIP卡失败: " + err.Error())
			return nil, errors.New("创建VIP卡失败")
		}
	}
	return vipcard, nil
}

func (s *VipcardServ) FindCustomerByTel(tel string) (*model.TaCustomer, error) {
	customer, err := s.cdao.GetCustomerByPhone(tel)
	if err != nil {
		utils.LogError("获取信息失败: " + err.Error())
		return nil, errors.New("获取信息失败")
	}
	return customer, err
}

// SaveCustomerVip 保存用户vip
func (s *VipcardServ) SaveCustomerVip(vipchild *model.TaVipCardChild, createUser model.SysUser) error {
	customer, err := s.cdao.GetCustomerByPhone(vipchild.CustomerTel)
	if err != nil {
		utils.LogError("激活失败: " + err.Error())
		return errors.New("激活失败")
	}
	var customerVip = model.TaCustomerVip{
		CustomerId:     customer.CustomerId,
		VipCardId:      vipchild.VipCardId,
		VipCardChildId: vipchild.VipCardChildId,
		CustomerTel:    customer.Phone,
		Amount:         vipchild.Amount,
		BeginDate:      vipchild.BeginDate,
		EndDate:        vipchild.EndDate,
		Balance:        vipchild.Amount,
		CaseId:         vipchild.CaseId,
		OrgId:          vipchild.OrgId,
		SalesId:        vipchild.SalesId,
		SalesName:      vipchild.SalesName,
		PayAmount:      "0",
	}
	info, err := s.dao.AddCustomerVip(customerVip)
	if err != nil {
		utils.LogError("保存用户vip卡失败: " + err.Error())
		return errors.New("激活失败")
	}
	// vip卡明细操作
	var change = model.TaCustomerVipChange{
		CustomerVipId: info.CustomerVipId,
		CustomerId:    info.CustomerId,
		OrgId:         info.OrgId,
		CaseId:        info.CaseId,
		ChangeType:    models.CONSUME_COINCHG,
		ChangeSource:  models.ACCSOURCE_RECHARGE,
		SourceId:      vipchild.VipCardChildId,
		SourceName:    "充值",
		Amount:        info.Amount,
		FloatType:     models.ACCOUNT_INCOME,
		CreateUser:    createUser.UserId,
	}
	err = s.dao.InsertCustomerVipChange(change)
	if err != nil {
		utils.LogError("保存用户vip卡明细信息失败: " + err.Error())
		return errors.New("激活失败")
	}

	return err
}

// CustomerCharge 给用户充值
func (s *VipcardServ) CustomerCharge(vipchild *model.TaVipCardChild, createUser model.SysUser) error {
	customer, err := s.cdao.GetCustomerByPhone(vipchild.CustomerTel)
	if err != nil {
		utils.LogError("充值失败: " + err.Error())
		return errors.New("充值失败")
	}
	customerId := customer.CustomerId
	account, err := s.cdao.GetAccountByCust(customerId)
	if err != nil {
		utils.LogError("充值失败: " + err.Error())
		return errors.New("充值失败")
	}
	vipcard, err := s.dao.GetVipCardById(vipchild.VipCardId)
	if err != nil {
		utils.LogError("充值失败: " + err.Error())
		return errors.New("充值失败")
	}
	var record model.TaAccountChange
	record.CustomerId = customerId
	record.Amount = vipcard.CardAmount
	record.CaseId = vipcard.CaseId
	record.CustomerName = customer.CustomerName
	record.ChangeType = models.CONSUME_COINCHG
	record.ChangeSource = models.ACCSOURCE_RECHARGE
	record.SourceId = vipchild.VipCardChildId
	record.SourceName = vipcard.VipCardName
	record.CreateUser = createUser.UserId
	record.FloatType = models.ACCOUNT_INCOME
	record.AccountId = account.AccountId
	err = s.cdao.InsertAccountRecords(&record)
	if err != nil {
		utils.LogError("充值失败: " + err.Error())
		return errors.New("充值失败")
	}
	return nil
}

// GetVipChildExcel 获取VIP卡列表
func (s *VipcardServ) GetVipChildExcel(caseids, cardNo, sellerName, userName string) ([]vipcard.VipChild, error) {

	vipchild, err := s.dao.GetVipCardListExcel(caseids, cardNo, sellerName, userName)
	if err != nil {
		utils.LogError("获取VIP卡信息失败: " + err.Error())
		return nil, errors.New("获取VIP卡信息失败")
	}

	return vipchild, err
}

// GetCustomerVips 获取用户vip卡信息
func (s *VipcardServ) GetCustomerVips() ([]vipcard.CustomerVipWithChange, error) {
	customer := s.ctx.Get("customer").(model.TaCustomer)
	vips, err := s.dao.GetCustomerVips(customer.CustomerId)
	if err != nil {
		utils.LogError("获取VIP卡信息失败: " + err.Error())
		return nil, errors.New("获取VIP卡信息失败")
	}

	return vips, err
}
